@@ -22,6 +22,7 @@
 #include "Kokkos_HelperPredicates.hpp"
 #include "Kokkos_ValueWrapperForNoNeutralElement.hpp"
 #include "Kokkos_IdentityReferenceUnaryFunctor.hpp"
+#include "Kokkos_FunctorsForExclusiveScan.hpp"
 #include <std_algorithms/Kokkos_TransformExclusiveScan.hpp>
 #include <std_algorithms/Kokkos_Distance.hpp>
 #include <string>
@@ -30,125 +31,15 @@ namespace Kokkos {
 namespace Experimental {
 namespace Impl {
 
-template <class ExeSpace, class IndexType, class ValueType, class FirstFrom,
-          class FirstDest>
-struct ExclusiveScanDefaultFunctorForKnownNeutralElement {
-  using execution_space = ExeSpace;
-
-  ValueType m_init_value;
-  FirstFrom m_first_from;
-  FirstDest m_first_dest;
-
-  KOKKOS_FUNCTION
-  ExclusiveScanDefaultFunctorForKnownNeutralElement(ValueType init,
-                                                    FirstFrom first_from,
-                                                    FirstDest first_dest)
-      : m_init_value(std::move(init)),
-        m_first_from(std::move(first_from)),
-        m_first_dest(std::move(first_dest)) {}
-
-  KOKKOS_FUNCTION
-  void operator()(const IndexType i, ValueType& update,
-                  const bool final_pass) const {
-    if (final_pass) m_first_dest[i] = update + m_init_value;
-    update += m_first_from[i];
-  }
-};
-
-template <class ExeSpace, class IndexType, class ValueType, class FirstFrom,
-          class FirstDest>
-struct ExclusiveScanDefaultFunctor {
-  using execution_space = ExeSpace;
-  using value_type =
-      ::Kokkos::Experimental::Impl::ValueWrapperForNoNeutralElement<ValueType>;
-
-  ValueType m_init_value;
-  FirstFrom m_first_from;
-  FirstDest m_first_dest;
-
-  KOKKOS_FUNCTION
-  ExclusiveScanDefaultFunctor(ValueType init, FirstFrom first_from,
-                              FirstDest first_dest)
-      : m_init_value(std::move(init)),
-        m_first_from(std::move(first_from)),
-        m_first_dest(std::move(first_dest)) {}
-
-  KOKKOS_FUNCTION
-  void operator()(const IndexType i, value_type& update,
-                  const bool final_pass) const {
-    if (final_pass) {
-      if (i == 0) {
-        m_first_dest[i] = m_init_value;
-      } else {
-        m_first_dest[i] = update.val + m_init_value;
-      }
-    }
-
-    const auto tmp = value_type{m_first_from[i], false};
-    this->join(update, tmp);
-  }
-
-  KOKKOS_FUNCTION
-  void init(value_type& update) const {
-    update.val        = {};
-    update.is_initial = true;
-  }
-
-  KOKKOS_FUNCTION
-  void join(value_type& update, const value_type& input) const {
-    if (update.is_initial) {
-      update.val        = input.val;
-      update.is_initial = false;
-    } else {
-      update.val = update.val + input.val;
-    }
-  }
-};
-
-template <class ExecutionSpace, class InputIteratorType,
-          class OutputIteratorType, class ValueType, class BinaryOpType>
-OutputIteratorType exclusive_scan_custom_op_impl(
-    const std::string& label, const ExecutionSpace& ex,
-    InputIteratorType first_from, InputIteratorType last_from,
-    OutputIteratorType first_dest, ValueType init_value, BinaryOpType bop) {
-  // checks
-  Impl::static_assert_random_access_and_accessible(ex, first_from, first_dest);
-  Impl::static_assert_iterators_have_matching_difference_type(first_from,
-                                                              first_dest);
-  Impl::expect_valid_range(first_from, last_from);
-
-  // aliases
-  using index_type    = typename InputIteratorType::difference_type;
-  using unary_op_type = StdNumericScanIdentityReferenceUnaryFunctor<ValueType>;
-  using func_type =
-      TransformExclusiveScanFunctor<ExecutionSpace, index_type, ValueType,
-                                    InputIteratorType, OutputIteratorType,
-                                    BinaryOpType, unary_op_type>;
-
-  // run
-  const auto num_elements =
-      Kokkos::Experimental::distance(first_from, last_from);
-  ::Kokkos::parallel_scan(
-      label, RangePolicy<ExecutionSpace>(ex, 0, num_elements),
-      func_type(init_value, first_from, first_dest, bop, unary_op_type()));
-  ex.fence("Kokkos::exclusive_scan_custom_op: fence after operation");
-
-  // return
-  return first_dest + num_elements;
-}
-
-template <typename ValueType>
-using ex_scan_has_reduction_identity_sum_t =
-    decltype(Kokkos::reduction_identity<ValueType>::sum());
-
+//
+// exespace impl
+//
 template <class ExecutionSpace, class InputIteratorType,
           class OutputIteratorType, class ValueType>
-OutputIteratorType exclusive_scan_default_op_impl(const std::string& label,
-                                                  const ExecutionSpace& ex,
-                                                  InputIteratorType first_from,
-                                                  InputIteratorType last_from,
-                                                  OutputIteratorType first_dest,
-                                                  ValueType init_value) {
+OutputIteratorType exclusive_scan_default_op_exespace_impl(
+    const std::string& label, const ExecutionSpace& ex,
+    InputIteratorType first_from, InputIteratorType last_from,
+    OutputIteratorType first_dest, ValueType init_value) {
   // checks
   Impl::static_assert_random_access_and_accessible(ex, first_from, first_dest);
   Impl::static_assert_iterators_have_matching_difference_type(first_from,
@@ -182,17 +73,122 @@ OutputIteratorType exclusive_scan_default_op_impl(const std::string& label,
       ExclusiveScanDefaultFunctorForKnownNeutralElement<
           ExecutionSpace, index_type, ValueType, InputIteratorType,
           OutputIteratorType>,
-      ExclusiveScanDefaultFunctor<ExecutionSpace, index_type, ValueType,
-                                  InputIteratorType, OutputIteratorType>>;
+      ExclusiveScanDefaultFunctorWithValueWrapper<ExecutionSpace, index_type,
+                                                  ValueType, InputIteratorType,
+                                                  OutputIteratorType>>;
+
+  // run
+  const auto num_elements =
+      Kokkos::Experimental::distance(first_from, last_from);
+  ::Kokkos::parallel_scan(
+      label, RangePolicy<ExecutionSpace>(ex, 0, num_elements),
+      func_type(std::move(init_value), first_from, first_dest));
+
+  ex.fence("Kokkos::exclusive_scan_default_op: fence after operation");
+
+  return first_dest + num_elements;
+}
+
+template <class ExecutionSpace, class InputIteratorType,
+          class OutputIteratorType, class ValueType, class BinaryOpType>
+OutputIteratorType exclusive_scan_custom_op_exespace_impl(
+    const std::string& label, const ExecutionSpace& ex,
+    InputIteratorType first_from, InputIteratorType last_from,
+    OutputIteratorType first_dest, ValueType init_value, BinaryOpType bop) {
+  // checks
+  Impl::static_assert_random_access_and_accessible(ex, first_from, first_dest);
+  Impl::static_assert_iterators_have_matching_difference_type(first_from,
+                                                              first_dest);
+  Impl::expect_valid_range(first_from, last_from);
+
+  // aliases
+  using index_type    = typename InputIteratorType::difference_type;
+  using unary_op_type = StdNumericScanIdentityReferenceUnaryFunctor;
+  using func_type     = TransformExclusiveScanFunctorWithValueWrapper<
+      ExecutionSpace, index_type, ValueType, InputIteratorType,
+      OutputIteratorType, BinaryOpType, unary_op_type>;
 
   // run
   const auto num_elements =
       Kokkos::Experimental::distance(first_from, last_from);
   ::Kokkos::parallel_scan(label,
                           RangePolicy<ExecutionSpace>(ex, 0, num_elements),
-                          func_type(init_value, first_from, first_dest));
+                          func_type(std::move(init_value), first_from,
+                                    first_dest, bop, unary_op_type()));
+  ex.fence("Kokkos::exclusive_scan_custom_op: fence after operation");
 
-  ex.fence("Kokkos::exclusive_scan_default_op: fence after operation");
+  // return
+  return first_dest + num_elements;
+}
+
+//
+// team impl
+//
+template <class TeamHandleType, class InputIteratorType,
+          class OutputIteratorType, class ValueType>
+KOKKOS_FUNCTION OutputIteratorType exclusive_scan_default_op_team_impl(
+    const TeamHandleType& teamHandle, InputIteratorType first_from,
+    InputIteratorType last_from, OutputIteratorType first_dest,
+    ValueType init_value) {
+  // checks
+  Impl::static_assert_random_access_and_accessible(teamHandle, first_from,
+                                                   first_dest);
+  Impl::static_assert_iterators_have_matching_difference_type(first_from,
+                                                              first_dest);
+  Impl::expect_valid_range(first_from, last_from);
+
+  static_assert(
+      ::Kokkos::is_detected_v<ex_scan_has_reduction_identity_sum_t, ValueType>,
+      "The team-level impl of Kokkos::Experimental::exclusive_scan currently "
+      "does not support types without reduction identity");
+
+  // aliases
+  using exe_space  = typename TeamHandleType::execution_space;
+  using index_type = typename InputIteratorType::difference_type;
+  using func_type  = ExclusiveScanDefaultFunctorForKnownNeutralElement<
+      exe_space, index_type, ValueType, InputIteratorType, OutputIteratorType>;
+
+  const auto num_elements =
+      Kokkos::Experimental::distance(first_from, last_from);
+  ::Kokkos::parallel_scan(
+      TeamThreadRange(teamHandle, 0, num_elements),
+      func_type(std::move(init_value), first_from, first_dest));
+  teamHandle.team_barrier();
+  return first_dest + num_elements;
+}
+
+template <class TeamHandleType, class InputIteratorType,
+          class OutputIteratorType, class ValueType, class BinaryOpType>
+KOKKOS_FUNCTION OutputIteratorType exclusive_scan_custom_op_team_impl(
+    const TeamHandleType& teamHandle, InputIteratorType first_from,
+    InputIteratorType last_from, OutputIteratorType first_dest,
+    ValueType init_value, BinaryOpType bop) {
+  // checks
+  Impl::static_assert_random_access_and_accessible(teamHandle, first_from,
+                                                   first_dest);
+  Impl::static_assert_iterators_have_matching_difference_type(first_from,
+                                                              first_dest);
+  Impl::expect_valid_range(first_from, last_from);
+
+  static_assert(
+      ::Kokkos::is_detected_v<ex_scan_has_reduction_identity_sum_t, ValueType>,
+      "The team-level impl of Kokkos::Experimental::exclusive_scan currently "
+      "does not support types without reduction identity");
+
+  // aliases
+  using exe_space     = typename TeamHandleType::execution_space;
+  using unary_op_type = StdNumericScanIdentityReferenceUnaryFunctor;
+  using index_type    = typename InputIteratorType::difference_type;
+  using func_type     = TransformExclusiveScanFunctorWithoutValueWrapper<
+      exe_space, index_type, ValueType, InputIteratorType, OutputIteratorType,
+      BinaryOpType, unary_op_type>;
+
+  const auto num_elements =
+      Kokkos::Experimental::distance(first_from, last_from);
+  ::Kokkos::parallel_scan(TeamThreadRange(teamHandle, 0, num_elements),
+                          func_type(std::move(init_value), first_from,
+                                    first_dest, bop, unary_op_type()));
+  teamHandle.team_barrier();
 
   return first_dest + num_elements;
 }
