@@ -42,10 +42,9 @@ struct StdForEachFunctor {
 };
 
 template <class HandleType, class IteratorType, class UnaryFunctorType>
-UnaryFunctorType for_each_exespace_impl(const std::string& label,
-                                        const HandleType& handle,
-                                        IteratorType first, IteratorType last,
-                                        UnaryFunctorType functor) {
+void for_each_exespace_impl(const std::string& label, const HandleType& handle,
+                            IteratorType first, IteratorType last,
+                            UnaryFunctorType functor) {
   // checks
   Impl::static_assert_random_access_and_accessible(handle, first);
   Impl::expect_valid_range(first, last);
@@ -56,8 +55,6 @@ UnaryFunctorType for_each_exespace_impl(const std::string& label,
       label, RangePolicy<HandleType>(handle, 0, num_elements),
       StdForEachFunctor<IteratorType, UnaryFunctorType>(first, functor));
   handle.fence("Kokkos::for_each: fence after operation");
-
-  return functor;
 }
 
 template <class ExecutionSpace, class IteratorType, class SizeType,
@@ -75,7 +72,7 @@ IteratorType for_each_n_exespace_impl(const std::string& label,
   }
 
   for_each_exespace_impl(label, ex, first, last, std::move(functor));
-  // no neeed to fence since for_each_exespace_impl fences already
+  // no need to fence since for_each_exespace_impl fences already
 
   return last;
 }
@@ -84,9 +81,9 @@ IteratorType for_each_n_exespace_impl(const std::string& label,
 // team impl
 //
 template <class TeamHandleType, class IteratorType, class UnaryFunctorType>
-KOKKOS_FUNCTION UnaryFunctorType
-for_each_team_impl(const TeamHandleType& teamHandle, IteratorType first,
-                   IteratorType last, UnaryFunctorType functor) {
+KOKKOS_FUNCTION void for_each_team_impl(const TeamHandleType& teamHandle,
+                                        IteratorType first, IteratorType last,
+                                        UnaryFunctorType functor) {
   // checks
   Impl::static_assert_random_access_and_accessible(teamHandle, first);
   Impl::expect_valid_range(first, last);
@@ -96,7 +93,6 @@ for_each_team_impl(const TeamHandleType& teamHandle, IteratorType first,
       TeamThreadRange(teamHandle, 0, num_elements),
       StdForEachFunctor<IteratorType, UnaryFunctorType>(first, functor));
   teamHandle.team_barrier();
-  return functor;
 }
 
 template <class TeamHandleType, class IteratorType, class SizeType,
@@ -113,7 +109,7 @@ for_each_n_team_impl(const TeamHandleType& teamHandle, IteratorType first,
   }
 
   for_each_team_impl(teamHandle, first, last, std::move(functor));
-  // no neeed to fence since for_each_team_impl fences already
+  // no need to fence since for_each_team_impl fences already
 
   return last;
 }
