@@ -18,9 +18,15 @@
 #define KOKKOS_ALGORITHMS_UNITTESTS_TEST_STD_ALGOS_COMMON_HPP
 
 #include <gtest/gtest.h>
-#include <Kokkos_Core.hpp>
-#include <Kokkos_StdAlgorithms.hpp>
+#include <Kokkos_Macros.hpp>
+#ifdef KOKKOS_ENABLE_EXPERIMENTAL_CXX20_MODULES
+import kokkos.random;
+import kokkos.std_algorithms;
+#else
 #include <Kokkos_Random.hpp>
+#include <Kokkos_StdAlgorithms.hpp>
+#endif
+#include <Kokkos_Core.hpp>
 #include <TestStdAlgorithmsHelperFunctors.hpp>
 #include <utility>
 #include <numeric>
@@ -679,10 +685,10 @@ struct std_algorithms_test : public ::testing::Test {
 
 struct CustomValueType {
   KOKKOS_INLINE_FUNCTION
-  CustomValueType(){};
+  CustomValueType() {}
 
   KOKKOS_INLINE_FUNCTION
-  CustomValueType(value_type val) : value(val){};
+  CustomValueType(value_type val) : value(val) {}
 
   KOKKOS_INLINE_FUNCTION
   CustomValueType(const CustomValueType& other) { this->value = other.value; }
