@@ -57,14 +57,7 @@ struct TestFunctorA {
     const auto myRowIndex = member.league_rank();
     auto myRowViewFrom = Kokkos::subview(m_dataView, myRowIndex, Kokkos::ALL());
     const auto val     = m_greaterThanValuesView(myRowIndex);
-    // FIXME_INTEL
-#if defined(KOKKOS_COMPILER_INTEL) && (1900 == KOKKOS_COMPILER_INTEL)
-    GreaterEqualFunctor<
-        typename GreaterThanValuesViewType::non_const_value_type>
-        unaryPred{val};
-#else
     GreaterEqualFunctor unaryPred{val};
-#endif
     ptrdiff_t resultDist = 0;
 
     switch (m_apiPick) {
@@ -185,12 +178,7 @@ void test_A(const bool predicatesReturnTrue, std::size_t numTeams,
     const auto rowFromBegin = KE::cbegin(rowFrom);
     const auto rowFromEnd   = KE::cend(rowFrom);
     const auto val          = greaterEqualValuesView_h(i);
-    // FIXME_INTEL
-#if defined(KOKKOS_COMPILER_INTEL) && (1900 == KOKKOS_COMPILER_INTEL)
-    const GreaterEqualFunctor<ValueType> unaryPred{val};
-#else
     const GreaterEqualFunctor unaryPred{val};
-#endif
 
     auto it = std::find_if(rowFromBegin, rowFromEnd, unaryPred);
 
