@@ -15,6 +15,10 @@
 //@HEADER
 
 #include <TestStdAlgorithmsCommon.hpp>
+#ifdef KOKKOS_ENABLE_EXPERIMENTAL_CXX20_MODULES
+#include <std_algorithms/impl/Kokkos_IdentityReferenceUnaryFunctor.hpp>
+#include <std_algorithms/impl/Kokkos_FunctorsForExclusiveScan.hpp>
+#endif
 #include <utility>
 #include <iomanip>
 
@@ -71,24 +75,24 @@ void fill_view(ViewType dest_view, const std::string& name) {
   }
 
   else if (name == "one-element") {
-    assert(v_h.extent(0) == 1);
+    KOKKOS_ASSERT(v_h.extent(0) == 1);
     v_h(0) = static_cast<value_type>(1);
   }
 
   else if (name == "two-elements-a") {
-    assert(v_h.extent(0) == 2);
+    KOKKOS_ASSERT(v_h.extent(0) == 2);
     v_h(0) = static_cast<value_type>(1);
     v_h(1) = static_cast<value_type>(2);
   }
 
   else if (name == "two-elements-b") {
-    assert(v_h.extent(0) == 2);
+    KOKKOS_ASSERT(v_h.extent(0) == 2);
     v_h(0) = static_cast<value_type>(2);
     v_h(1) = static_cast<value_type>(-1);
   }
 
   else if (name == "small-a") {
-    assert(v_h.extent(0) == 9);
+    KOKKOS_ASSERT(v_h.extent(0) == 9);
     v_h(0) = static_cast<value_type>(3);
     v_h(1) = static_cast<value_type>(1);
     v_h(2) = static_cast<value_type>(4);
@@ -101,7 +105,7 @@ void fill_view(ViewType dest_view, const std::string& name) {
   }
 
   else if (name == "small-b") {
-    assert(v_h.extent(0) >= 6);
+    KOKKOS_ASSERT(v_h.extent(0) >= 6);
     for (std::size_t i = 0; i < ext; ++i) {
       v_h(i) = randObj();
     }
@@ -115,7 +119,7 @@ void fill_view(ViewType dest_view, const std::string& name) {
   }
 
   else {
-    throw std::runtime_error("invalid choice");
+    FAIL() << "invalid choice";
   }
 
   Kokkos::deep_copy(aux_view, v_h);
@@ -161,7 +165,7 @@ void verify_data(ViewType1 data_view,  // contains data
       create_mirror_view_and_copy(Kokkos::HostSpace(), test_view_dc);
   if (test_view_h.extent(0) > 0) {
     for (std::size_t i = 0; i < test_view_h.extent(0); ++i) {
-      if (std::is_same<gold_view_value_type, int>::value) {
+      if (std::is_same_v<gold_view_value_type, int>) {
         ASSERT_EQ(gold_h(i), test_view_h(i));
       } else {
         const auto error = std::abs(gold_h(i) - test_view_h(i));
@@ -344,8 +348,7 @@ TEST(std_algorithms_numeric_ops_test, transform_exclusive_scan_functor) {
   using view_type = Kokkos::View<int*, exespace>;
   view_type dummy_view("dummy_view", 0);
   using unary_op_type =
-      Kokkos::Experimental::Impl::StdNumericScanIdentityReferenceUnaryFunctor<
-          int>;
+      Kokkos::Experimental::Impl::StdNumericScanIdentityReferenceUnaryFunctor;
   using functor_type =
       Kokkos::Experimental::Impl::TransformExclusiveScanFunctorWithValueWrapper<
           exespace, int, int, view_type, view_type, MultiplyFunctor<int>,
