@@ -17,12 +17,22 @@
 #ifndef KOKKOS_TEST_PERFTEST_CAT_HPP
 #define KOKKOS_TEST_PERFTEST_CAT_HPP
 
-#include <gtest/gtest.h>
-
 namespace Test {
 
-extern int command_line_num_args(int n = 0);
-extern const char* command_line_arg(int k, char** input_args = nullptr);
+inline int command_line_num_args(int n = 0) {
+  static int n_args = 0;
+  if (n > 0) n_args = n;
+  return n_args;
+}
+
+inline const char* command_line_arg(int k, char** input_args = nullptr) {
+  static char** args;
+  if (input_args != nullptr) args = input_args;
+  if (command_line_num_args() > k)
+    return args[k];
+  else
+    return nullptr;
+}
 
 }  // namespace Test
 
