@@ -23,11 +23,25 @@ BENCHMARK(ViewFill_Rank1<Kokkos::LayoutLeft>)
     ->Arg(N)
     ->UseManualTime();
 
+#ifdef KOKKOS_HAS_SHARED_SPACE
+BENCHMARK(ViewFill_Rank1<Kokkos::LayoutLeft, Kokkos::SharedSpace>)
+    ->ArgName("N")
+    ->Arg(N)
+    ->UseManualTime();
+#endif
+
 BENCHMARK(ViewFill_Rank1<Kokkos::LayoutRight>)
     ->ArgName("N")
     ->Arg(N)
     ->UseManualTime();
 
+#ifdef KOKKOS_HAS_SHARED_SPACE
+BENCHMARK(ViewFill_Rank1<Kokkos::LayoutRight, Kokkos::SharedSpace>)
+    ->ArgName("N")
+    ->Arg(N)
+    ->UseManualTime();
+#endif
+
 BENCHMARK(ViewFill_Rank2<Kokkos::LayoutLeft>)
     ->ArgName("N")
     ->Arg(N)
