@@ -130,31 +130,33 @@ inline void check_shmem_request(CudaInternal const* cuda_instance, int shmem) {
 // KernelFuncPtr does not necessarily contain that type information.
 template <class DriverType, class LaunchBounds, class KernelFuncPtr>
 const cudaFuncAttributes& get_cuda_kernel_func_attributes(
-    int cuda_device, const KernelFuncPtr& func) {
+    const CudaInternal* cuda_instance, const KernelFuncPtr& func) {
   // Only call cudaFuncGetAttributes once for each unique kernel
   // by leveraging static variable initialization rules
+  const auto cuda_device = cuda_instance->m_cudaDev;
   static std::map<int, cudaFuncAttributes> func_attr;
   if (func_attr.find(cuda_device) == func_attr.end()) {
     cudaFuncAttributes attr;
-    KOKKOS_IMPL_CUDA_SAFE_CALL(cudaSetDevice(cuda_device));
-    KOKKOS_IMPL_CUDA_SAFE_CALL(cudaFuncGetAttributes(&attr, func));
+    KOKKOS_IMPL_CUDA_SAFE_CALL(
+        (cuda_instance->cuda_func_get_attributes_wrapper(&attr, func)));
     func_attr.emplace(cuda_device, attr);
   }
   return func_attr[cuda_device];
 }
 
 template <class DriverType, class LaunchBounds, class KernelFuncPtr>
-inline void configure_shmem_preference(const int cuda_device,
+inline void configure_shmem_preference(const CudaInternal* cuda_instance,
                                        const KernelFuncPtr& func,
-                                       const cudaDeviceProp& device_props,
                                        const size_t block_size, int& shmem,
                                        const size_t occupancy) {
 #ifndef KOKKOS_ARCH_KEPLER
 
   const auto& func_attr =
-      get_cuda_kernel_func_attributes<DriverType, LaunchBounds>(cuda_device,
+      get_cuda_kernel_func_attributes<DriverType, LaunchBounds>(cuda_instance,
                                                                 func);
 
+  const auto device_props = cuda_instance->m_deviceProp;
+
   // Compute limits for number of blocks due to registers/SM
   const size_t regs_per_sm     = device_props.regsPerMultiprocessor;
   const size_t regs_per_thread = func_attr.numRegs;
@@ -219,11 +221,9 @@ inline void configure_shmem_preference(const int cuda_device,
   if (carveout > 100) carveout = 100;
 
   // Set the carveout, but only call it once per kernel or when it changes
-  // FIXME_CUDA_MULTIPLE_DEVICES
   auto set_cache_config = [&] {
-    KOKKOS_IMPL_CUDA_SAFE_CALL(
-        (CudaInternal::singleton().cuda_func_set_attribute_wrapper(
-            func, cudaFuncAttributePreferredSharedMemoryCarveout, carveout)));
+    KOKKOS_IMPL_CUDA_SAFE_CALL((cuda_instance->cuda_func_set_attribute_wrapper(
+        func, cudaFuncAttributePreferredSharedMemoryCarveout, carveout)));
     return carveout;
   };
   // Store the value in a static variable so we only reset if needed
@@ -252,56 +252,50 @@ inline void configure_shmem_preference(const int cuda_device,
 
 template <class DriverType>
 struct DeduceCudaLaunchMechanism {
-  constexpr static const Kokkos::Experimental::WorkItemProperty::
-      HintLightWeight_t light_weight =
-          Kokkos::Experimental::WorkItemProperty::HintLightWeight;
-  constexpr static const Kokkos::Experimental::WorkItemProperty::
-      HintHeavyWeight_t heavy_weight =
-          Kokkos::Experimental::WorkItemProperty::HintHeavyWeight;
-  constexpr static const typename DriverType::Policy::work_item_property
-      property = typename DriverType::Policy::work_item_property();
+  constexpr static auto light_weight =
+      Kokkos::Experimental::WorkItemProperty::HintLightWeight;
+  constexpr static auto heavy_weight =
+      Kokkos::Experimental::WorkItemProperty::HintHeavyWeight;
+  constexpr static typename DriverType::Policy::work_item_property property{};
 
-  static constexpr const Experimental::CudaLaunchMechanism
-      valid_launch_mechanism =
-          // BuildValidMask
+  static constexpr CudaLaunchMechanism valid_launch_mechanism =
+      // BuildValidMask
       (sizeof(DriverType) < CudaTraits::KernelArgumentLimit
-           ? Experimental::CudaLaunchMechanism::LocalMemory
-           : Experimental::CudaLaunchMechanism::Default) |
+           ? CudaLaunchMechanism::LocalMemory
+           : CudaLaunchMechanism::Default) |
       (sizeof(DriverType) < CudaTraits::ConstantMemoryUsage
-           ? Experimental::CudaLaunchMechanism::ConstantMemory
-           : Experimental::CudaLaunchMechanism::Default) |
-      Experimental::CudaLaunchMechanism::GlobalMemory;
+           ? CudaLaunchMechanism::ConstantMemory
+           : CudaLaunchMechanism::Default) |
+      CudaLaunchMechanism::GlobalMemory;
 
-  static constexpr const Experimental::CudaLaunchMechanism
-      requested_launch_mechanism =
-          (((property & light_weight) == light_weight)
-               ? Experimental::CudaLaunchMechanism::LocalMemory
-               : Experimental::CudaLaunchMechanism::ConstantMemory) |
-          Experimental::CudaLaunchMechanism::GlobalMemory;
+  static constexpr CudaLaunchMechanism requested_launch_mechanism =
+      (((property & light_weight) == light_weight)
+           ? CudaLaunchMechanism::LocalMemory
+           : CudaLaunchMechanism::ConstantMemory) |
+      CudaLaunchMechanism::GlobalMemory;
 
-  static constexpr const Experimental::CudaLaunchMechanism
-      default_launch_mechanism =
-          // BuildValidMask
+  static constexpr CudaLaunchMechanism default_launch_mechanism =
+      // BuildValidMask
       (sizeof(DriverType) < CudaTraits::ConstantMemoryUseThreshold)
-          ? Experimental::CudaLaunchMechanism::LocalMemory
+          ? CudaLaunchMechanism::LocalMemory
           : ((sizeof(DriverType) < CudaTraits::ConstantMemoryUsage)
-                 ? Experimental::CudaLaunchMechanism::ConstantMemory
-                 : Experimental::CudaLaunchMechanism::GlobalMemory);
+                 ? CudaLaunchMechanism::ConstantMemory
+                 : CudaLaunchMechanism::GlobalMemory);
 
   //              None                LightWeight    HeavyWeight
   // F<UseT       LCG LCG L  L        LCG  LG L  L    LCG  CG L  C
   // UseT<F<KAL   LCG LCG C  C        LCG  LG C  L    LCG  CG C  C
   // Kal<F<CMU     CG LCG C  C         CG  LG C  G     CG  CG C  C
   // CMU<F          G LCG G  G          G  LG G  G      G  CG G  G
-  static constexpr const Experimental::CudaLaunchMechanism launch_mechanism =
+  static constexpr CudaLaunchMechanism launch_mechanism =
       ((property & light_weight) == light_weight)
           ? (sizeof(DriverType) < CudaTraits::KernelArgumentLimit
-                 ? Experimental::CudaLaunchMechanism::LocalMemory
-                 : Experimental::CudaLaunchMechanism::GlobalMemory)
+                 ? CudaLaunchMechanism::LocalMemory
+                 : CudaLaunchMechanism::GlobalMemory)
           : (((property & heavy_weight) == heavy_weight)
                  ? (sizeof(DriverType) < CudaTraits::ConstantMemoryUsage
-                        ? Experimental::CudaLaunchMechanism::ConstantMemory
-                        : Experimental::CudaLaunchMechanism::GlobalMemory)
+                        ? CudaLaunchMechanism::ConstantMemory
+                        : CudaLaunchMechanism::GlobalMemory)
                  : (default_launch_mechanism));
 };
 
@@ -315,11 +309,11 @@ struct DeduceCudaLaunchMechanism {
 // mechanisms
 
 template <class DriverType, class LaunchBounds,
-          Experimental::CudaLaunchMechanism LaunchMechanism>
+          CudaLaunchMechanism LaunchMechanism>
 struct CudaParallelLaunchKernelFunc;
 
 template <class DriverType, class LaunchBounds,
-          Experimental::CudaLaunchMechanism LaunchMechanism>
+          CudaLaunchMechanism LaunchMechanism>
 struct CudaParallelLaunchKernelInvoker;
 
 //------------------------------------------------------------------------------
@@ -329,7 +323,7 @@ template <class DriverType, unsigned int MaxThreadsPerBlock,
           unsigned int MinBlocksPerSM>
 struct CudaParallelLaunchKernelFunc<
     DriverType, Kokkos::LaunchBounds<MaxThreadsPerBlock, MinBlocksPerSM>,
-    Experimental::CudaLaunchMechanism::LocalMemory> {
+    CudaLaunchMechanism::LocalMemory> {
   static std::decay_t<decltype(cuda_parallel_launch_local_memory<
                                DriverType, MaxThreadsPerBlock, MinBlocksPerSM>)>
   get_kernel_func() {
@@ -339,9 +333,8 @@ struct CudaParallelLaunchKernelFunc<
 };
 
 template <class DriverType>
-struct CudaParallelLaunchKernelFunc<
-    DriverType, Kokkos::LaunchBounds<0, 0>,
-    Experimental::CudaLaunchMechanism::LocalMemory> {
+struct CudaParallelLaunchKernelFunc<DriverType, Kokkos::LaunchBounds<0, 0>,
+                                    CudaLaunchMechanism::LocalMemory> {
   static std::decay_t<decltype(cuda_parallel_launch_local_memory<DriverType>)>
   get_kernel_func() {
     return cuda_parallel_launch_local_memory<DriverType>;
@@ -351,13 +344,12 @@ struct CudaParallelLaunchKernelFunc<
 //------------------------------------------------------------------------------
 
 template <class DriverType, class LaunchBounds>
-struct CudaParallelLaunchKernelInvoker<
-    DriverType, LaunchBounds, Experimental::CudaLaunchMechanism::LocalMemory>
-    : CudaParallelLaunchKernelFunc<
-          DriverType, LaunchBounds,
-          Experimental::CudaLaunchMechanism::LocalMemory> {
-  using base_t = CudaParallelLaunchKernelFunc<
-      DriverType, LaunchBounds, Experimental::CudaLaunchMechanism::LocalMemory>;
+struct CudaParallelLaunchKernelInvoker<DriverType, LaunchBounds,
+                                       CudaLaunchMechanism::LocalMemory>
+    : CudaParallelLaunchKernelFunc<DriverType, LaunchBounds,
+                                   CudaLaunchMechanism::LocalMemory> {
+  using base_t = CudaParallelLaunchKernelFunc<DriverType, LaunchBounds,
+                                              CudaLaunchMechanism::LocalMemory>;
   static_assert(sizeof(DriverType) < CudaTraits::KernelArgumentLimit,
                 "Kokkos Error: Requested CudaLaunchLocalMemory with a Functor "
                 "larger than 4096 bytes.");
@@ -365,8 +357,12 @@ struct CudaParallelLaunchKernelInvoker<
   static void invoke_kernel(DriverType const& driver, dim3 const& grid,
                             dim3 const& block, int shmem,
                             CudaInternal const* cuda_instance) {
-    (base_t::get_kernel_func())<<<grid, block, shmem,
-                                  cuda_instance->get_stream()>>>(driver);
+    // Set cuda device before launching kernel
+    cuda_instance->set_cuda_device();
+
+    (base_t::
+         get_kernel_func())<<<grid, block, shmem, cuda_instance->m_stream>>>(
+        driver);
   }
 
   inline static void create_parallel_launch_graph_node(
@@ -429,7 +425,7 @@ template <class DriverType, unsigned int MaxThreadsPerBlock,
           unsigned int MinBlocksPerSM>
 struct CudaParallelLaunchKernelFunc<
     DriverType, Kokkos::LaunchBounds<MaxThreadsPerBlock, MinBlocksPerSM>,
-    Experimental::CudaLaunchMechanism::GlobalMemory> {
+    CudaLaunchMechanism::GlobalMemory> {
   static void* get_kernel_func() {
     return cuda_parallel_launch_global_memory<DriverType, MaxThreadsPerBlock,
                                               MinBlocksPerSM>;
@@ -437,9 +433,8 @@ struct CudaParallelLaunchKernelFunc<
 };
 
 template <class DriverType>
-struct CudaParallelLaunchKernelFunc<
-    DriverType, Kokkos::LaunchBounds<0, 0>,
-    Experimental::CudaLaunchMechanism::GlobalMemory> {
+struct CudaParallelLaunchKernelFunc<DriverType, Kokkos::LaunchBounds<0, 0>,
+                                    CudaLaunchMechanism::GlobalMemory> {
   static std::decay_t<decltype(cuda_parallel_launch_global_memory<DriverType>)>
   get_kernel_func() {
     return cuda_parallel_launch_global_memory<DriverType>;
@@ -449,14 +444,13 @@ struct CudaParallelLaunchKernelFunc<
 //------------------------------------------------------------------------------
 
 template <class DriverType, class LaunchBounds>
-struct CudaParallelLaunchKernelInvoker<
-    DriverType, LaunchBounds, Experimental::CudaLaunchMechanism::GlobalMemory>
-    : CudaParallelLaunchKernelFunc<
-          DriverType, LaunchBounds,
-          Experimental::CudaLaunchMechanism::GlobalMemory> {
-  using base_t = CudaParallelLaunchKernelFunc<
-      DriverType, LaunchBounds,
-      Experimental::CudaLaunchMechanism::GlobalMemory>;
+struct CudaParallelLaunchKernelInvoker<DriverType, LaunchBounds,
+                                       CudaLaunchMechanism::GlobalMemory>
+    : CudaParallelLaunchKernelFunc<DriverType, LaunchBounds,
+                                   CudaLaunchMechanism::GlobalMemory> {
+  using base_t =
+      CudaParallelLaunchKernelFunc<DriverType, LaunchBounds,
+                                   CudaLaunchMechanism::GlobalMemory>;
 
   static void invoke_kernel(DriverType const& driver, dim3 const& grid,
                             dim3 const& block, int shmem,
@@ -466,8 +460,13 @@ struct CudaParallelLaunchKernelInvoker<
 
     KOKKOS_IMPL_CUDA_SAFE_CALL((cuda_instance->cuda_memcpy_async_wrapper(
         driver_ptr, &driver, sizeof(DriverType), cudaMemcpyDefault)));
-    (base_t::get_kernel_func())<<<grid, block, shmem,
-                                  cuda_instance->get_stream()>>>(driver_ptr);
+
+    // Set cuda device before launching kernel
+    cuda_instance->set_cuda_device();
+
+    (base_t::
+         get_kernel_func())<<<grid, block, shmem, cuda_instance->m_stream>>>(
+        driver_ptr);
   }
 
   inline static void create_parallel_launch_graph_node(
@@ -488,8 +487,8 @@ struct CudaParallelLaunchKernelInvoker<
             driver.get_policy().impl_get_desired_occupancy().value();
         size_t block_size = static_cast<size_t>(block.x) * block.y * block.z;
         Impl::configure_shmem_preference<DriverType, LaunchBounds>(
-            cuda_instance->m_cudaDev, base_t::get_kernel_func(),
-            cuda_instance->m_deviceProp, block_size, shmem, desired_occupancy);
+            cuda_instance, base_t::get_kernel_func(), block_size, shmem,
+            desired_occupancy);
       }
 
       auto* driver_ptr = Impl::allocate_driver_storage_for_kernel(
@@ -543,7 +542,7 @@ template <class DriverType, unsigned int MaxThreadsPerBlock,
           unsigned int MinBlocksPerSM>
 struct CudaParallelLaunchKernelFunc<
     DriverType, Kokkos::LaunchBounds<MaxThreadsPerBlock, MinBlocksPerSM>,
-    Experimental::CudaLaunchMechanism::ConstantMemory> {
+    CudaLaunchMechanism::ConstantMemory> {
   static std::decay_t<decltype(cuda_parallel_launch_constant_memory<
                                DriverType, MaxThreadsPerBlock, MinBlocksPerSM>)>
   get_kernel_func() {
@@ -553,9 +552,8 @@ struct CudaParallelLaunchKernelFunc<
 };
 
 template <class DriverType>
-struct CudaParallelLaunchKernelFunc<
-    DriverType, Kokkos::LaunchBounds<0, 0>,
-    Experimental::CudaLaunchMechanism::ConstantMemory> {
+struct CudaParallelLaunchKernelFunc<DriverType, Kokkos::LaunchBounds<0, 0>,
+                                    CudaLaunchMechanism::ConstantMemory> {
   static std::decay_t<
       decltype(cuda_parallel_launch_constant_memory<DriverType>)>
   get_kernel_func() {
@@ -566,14 +564,13 @@ struct CudaParallelLaunchKernelFunc<
 //------------------------------------------------------------------------------
 
 template <class DriverType, class LaunchBounds>
-struct CudaParallelLaunchKernelInvoker<
-    DriverType, LaunchBounds, Experimental::CudaLaunchMechanism::ConstantMemory>
-    : CudaParallelLaunchKernelFunc<
-          DriverType, LaunchBounds,
-          Experimental::CudaLaunchMechanism::ConstantMemory> {
-  using base_t = CudaParallelLaunchKernelFunc<
-      DriverType, LaunchBounds,
-      Experimental::CudaLaunchMechanism::ConstantMemory>;
+struct CudaParallelLaunchKernelInvoker<DriverType, LaunchBounds,
+                                       CudaLaunchMechanism::ConstantMemory>
+    : CudaParallelLaunchKernelFunc<DriverType, LaunchBounds,
+                                   CudaLaunchMechanism::ConstantMemory> {
+  using base_t =
+      CudaParallelLaunchKernelFunc<DriverType, LaunchBounds,
+                                   CudaLaunchMechanism::ConstantMemory>;
   static_assert(sizeof(DriverType) < CudaTraits::ConstantMemoryUsage,
                 "Kokkos Error: Requested CudaLaunchConstantMemory with a "
                 "Functor larger than 32kB.");
@@ -585,8 +582,8 @@ struct CudaParallelLaunchKernelInvoker<
     // Wait until the previous kernel that uses the constant buffer is done
     std::lock_guard<std::mutex> lock(
         CudaInternal::constantMemMutexPerDevice[cuda_device]);
-    KOKKOS_IMPL_CUDA_SAFE_CALL((cuda_instance->cuda_event_synchronize_wrapper(
-        CudaInternal::constantMemReusablePerDevice[cuda_device])));
+    KOKKOS_IMPL_CUDA_SAFE_CALL(cudaEventSynchronize(
+        CudaInternal::constantMemReusablePerDevice[cuda_device]));
 
     // Copy functor (synchronously) to staging buffer in pinned host memory
     unsigned long* staging =
@@ -600,9 +597,12 @@ struct CudaParallelLaunchKernelInvoker<
             kokkos_impl_cuda_constant_memory_buffer, staging,
             sizeof(DriverType), 0, cudaMemcpyHostToDevice)));
 
+    // Set cuda device before launching kernel
+    cuda_instance->set_cuda_device();
+
     // Invoke the driver function on the device
-    (base_t::get_kernel_func())<<<grid, block, shmem,
-                                  cuda_instance->get_stream()>>>();
+    (base_t::
+         get_kernel_func())<<<grid, block, shmem, cuda_instance->m_stream>>>();
 
     // Record an event that says when the constant buffer can be reused
     KOKKOS_IMPL_CUDA_SAFE_CALL((cuda_instance->cuda_event_record_wrapper(
@@ -621,9 +621,9 @@ struct CudaParallelLaunchKernelInvoker<
     // somehow go and prove was not creating a dependency cycle, and I don't
     // even know if there's an efficient way to do that, let alone in the
     // structure we currenty have).
-    using global_launch_impl_t = CudaParallelLaunchKernelInvoker<
-        DriverType, LaunchBounds,
-        Experimental::CudaLaunchMechanism::GlobalMemory>;
+    using global_launch_impl_t =
+        CudaParallelLaunchKernelInvoker<DriverType, LaunchBounds,
+                                        CudaLaunchMechanism::GlobalMemory>;
     global_launch_impl_t::create_parallel_launch_graph_node(
         driver, grid, block, shmem, cuda_instance);
   }
@@ -639,12 +639,11 @@ struct CudaParallelLaunchKernelInvoker<
 // <editor-fold desc="CudaParallelLaunchImpl"> {{{1
 
 template <class DriverType, class LaunchBounds,
-          Experimental::CudaLaunchMechanism LaunchMechanism>
+          CudaLaunchMechanism LaunchMechanism>
 struct CudaParallelLaunchImpl;
 
 template <class DriverType, unsigned int MaxThreadsPerBlock,
-          unsigned int MinBlocksPerSM,
-          Experimental::CudaLaunchMechanism LaunchMechanism>
+          unsigned int MinBlocksPerSM, CudaLaunchMechanism LaunchMechanism>
 struct CudaParallelLaunchImpl<
     DriverType, Kokkos::LaunchBounds<MaxThreadsPerBlock, MinBlocksPerSM>,
     LaunchMechanism>
@@ -674,8 +673,8 @@ struct CudaParallelLaunchImpl<
         Impl::configure_shmem_preference<
             DriverType,
             Kokkos::LaunchBounds<MaxThreadsPerBlock, MinBlocksPerSM>>(
-            cuda_instance->m_cudaDev, base_t::get_kernel_func(),
-            cuda_instance->m_deviceProp, block_size, shmem, desired_occupancy);
+            cuda_instance, base_t::get_kernel_func(), block_size, shmem,
+            desired_occupancy);
       }
 
       desul::ensure_cuda_lock_arrays_on_device();
@@ -691,10 +690,11 @@ struct CudaParallelLaunchImpl<
     }
   }
 
-  static cudaFuncAttributes get_cuda_func_attributes(int cuda_device) {
+  static cudaFuncAttributes get_cuda_func_attributes(
+      const CudaInternal* cuda_instance) {
     return get_cuda_kernel_func_attributes<
         DriverType, Kokkos::LaunchBounds<MaxThreadsPerBlock, MinBlocksPerSM>>(
-        cuda_device, base_t::get_kernel_func());
+        cuda_instance, base_t::get_kernel_func());
   }
 };
 
@@ -705,14 +705,14 @@ struct CudaParallelLaunchImpl<
 // <editor-fold desc="CudaParallelLaunch"> {{{1
 
 template <class DriverType, class LaunchBounds = Kokkos::LaunchBounds<>,
-          Experimental::CudaLaunchMechanism LaunchMechanism =
+          CudaLaunchMechanism LaunchMechanism =
               DeduceCudaLaunchMechanism<DriverType>::launch_mechanism,
           bool DoGraph = DriverType::Policy::is_graph_kernel::value>
 struct CudaParallelLaunch;
 
 // General launch mechanism
 template <class DriverType, class LaunchBounds,
-          Experimental::CudaLaunchMechanism LaunchMechanism>
+          CudaLaunchMechanism LaunchMechanism>
 struct CudaParallelLaunch<DriverType, LaunchBounds, LaunchMechanism,
                           /* DoGraph = */ false>
     : CudaParallelLaunchImpl<DriverType, LaunchBounds, LaunchMechanism> {
@@ -726,7 +726,7 @@ struct CudaParallelLaunch<DriverType, LaunchBounds, LaunchMechanism,
 
 // Launch mechanism for creating graph nodes
 template <class DriverType, class LaunchBounds,
-          Experimental::CudaLaunchMechanism LaunchMechanism>
+          CudaLaunchMechanism LaunchMechanism>
 struct CudaParallelLaunch<DriverType, LaunchBounds, LaunchMechanism,
                           /* DoGraph = */ true>
     : CudaParallelLaunchImpl<DriverType, LaunchBounds, LaunchMechanism> {
