@@ -42,7 +42,7 @@ template <typename ParallelType, typename Policy, typename LaunchBounds>
 int max_tile_size_product_helper(const Policy& pol, const LaunchBounds&) {
   cudaFuncAttributes attr =
       CudaParallelLaunch<ParallelType, LaunchBounds>::get_cuda_func_attributes(
-          pol.space().cuda_device());
+          pol.space().impl_internal_space_instance());
   auto const& prop = pol.space().cuda_device_prop();
 
   // Limits due to registers/SM, MDRange doesn't have
@@ -77,9 +77,11 @@ class ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>, Kokkos::Cuda> {
   using array_index_type = typename Policy::array_index_type;
   using index_type       = typename Policy::index_type;
   using LaunchBounds     = typename Policy::launch_bounds;
+  using MaxGridSize      = Kokkos::Array<index_type, 3>;
 
   const FunctorType m_functor;
   const Policy m_rp;
+  const MaxGridSize m_max_grid_size;
 
  public:
   template <typename Policy, typename Functor>
@@ -89,84 +91,86 @@ class ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>, Kokkos::Cuda> {
   Policy const& get_policy() const { return m_rp; }
   inline __device__ void operator()() const {
     Kokkos::Impl::DeviceIterateTile<Policy::rank, Policy, FunctorType,
-                                    typename Policy::work_tag>(m_rp, m_functor)
+                                    MaxGridSize, typename Policy::work_tag>(
+        m_rp, m_functor, m_max_grid_size)
         .exec_range();
   }
 
   inline void execute() const {
     if (m_rp.m_num_tiles == 0) return;
-    // maximum number of blocks per grid as fetched by the API
-    [[maybe_unused]] const auto maxblocks_api =
-        m_rp.space().cuda_device_prop().maxGridSize;
 
-    // maximum numebr of blocks per grid hard-coded for unpacking on device
-    const int maxblocks[3] = {mdrange_max_blocks_x, mdrange_max_blocks,
-                              mdrange_max_blocks};
+    // maximum number of threads in each dimension of the block as fetched by
+    // the API
+    const auto max_threads_dim = m_rp.space().cuda_device_prop().maxThreadsDim;
 
-    // check hard-coded values are compatible with API values
-    KOKKOS_ASSERT(maxblocks[0] <= static_cast<int>(maxblocks_api[0]));
-    KOKKOS_ASSERT(maxblocks[1] <= static_cast<int>(maxblocks_api[1]));
-    KOKKOS_ASSERT(maxblocks[2] <= static_cast<int>(maxblocks_api[2]));
-
-    const auto maxthreads = m_rp.space().cuda_device_prop().maxThreadsDim;
-
-    [[maybe_unused]] const auto maxThreadsPerBlock =
+    // maximum total number of threads per block as fetched by the API
+    [[maybe_unused]] const auto max_threads_per_block =
         m_rp.space().cuda_device_prop().maxThreadsPerBlock;
+
     // make sure the Z dimension (it is less than x,y limits) isn't exceeded
     const auto clampZ = [&](const int input) {
-      return (input > maxthreads[2] ? maxthreads[2] : input);
+      return std::min(input, max_threads_dim[2]);
     };
+
     // make sure the block dimensions don't exceed the max number of threads
     // allowed
     const auto check_block_sizes = [&]([[maybe_unused]] const dim3& block) {
       KOKKOS_ASSERT(block.x > 0 &&
-                    block.x <= static_cast<unsigned int>(maxthreads[0]));
+                    block.x <= static_cast<unsigned int>(max_threads_dim[0]));
       KOKKOS_ASSERT(block.y > 0 &&
-                    block.y <= static_cast<unsigned int>(maxthreads[1]));
+                    block.y <= static_cast<unsigned int>(max_threads_dim[1]));
       KOKKOS_ASSERT(block.z > 0 &&
-                    block.z <= static_cast<unsigned int>(maxthreads[2]));
+                    block.z <= static_cast<unsigned int>(max_threads_dim[2]));
       KOKKOS_ASSERT(block.x * block.y * block.z <=
-                    static_cast<unsigned int>(maxThreadsPerBlock));
+                    static_cast<unsigned int>(max_threads_per_block));
     };
+
     // make sure the grid dimensions don't exceed the max number of blocks
     // allowed
     const auto check_grid_sizes = [&]([[maybe_unused]] const dim3& grid) {
       KOKKOS_ASSERT(grid.x > 0 &&
-                    grid.x <= static_cast<unsigned int>(maxblocks[0]));
+                    grid.x <= static_cast<unsigned int>(m_max_grid_size[0]));
       KOKKOS_ASSERT(grid.y > 0 &&
-                    grid.y <= static_cast<unsigned int>(maxblocks[1]));
+                    grid.y <= static_cast<unsigned int>(m_max_grid_size[1]));
       KOKKOS_ASSERT(grid.z > 0 &&
-                    grid.z <= static_cast<unsigned int>(maxblocks[2]));
+                    grid.z <= static_cast<unsigned int>(m_max_grid_size[2]));
     };
+
     if (RP::rank == 2) {
+      // id0 to threadIdx.x; id1 to threadIdx.y
       const dim3 block(m_rp.m_tile[0], m_rp.m_tile[1], 1);
       check_block_sizes(block);
+
       const dim3 grid(
           std::min<array_index_type>(
               (m_rp.m_upper[0] - m_rp.m_lower[0] + block.x - 1) / block.x,
-              maxblocks[0]),
+              m_max_grid_size[0]),
           std::min<array_index_type>(
               (m_rp.m_upper[1] - m_rp.m_lower[1] + block.y - 1) / block.y,
-              maxblocks[1]),
+              m_max_grid_size[1]),
           1);
       check_grid_sizes(grid);
+
       CudaParallelLaunch<ParallelFor, LaunchBounds>(
           *this, grid, block, 0, m_rp.space().impl_internal_space_instance());
     } else if (RP::rank == 3) {
+      // id0 to threadIdx.x; id1 to threadIdx.y; id2 to threadIdx.z
       const dim3 block(m_rp.m_tile[0], m_rp.m_tile[1], clampZ(m_rp.m_tile[2]));
       check_block_sizes(block);
+
       const dim3 grid(
           std::min<array_index_type>(
               (m_rp.m_upper[0] - m_rp.m_lower[0] + block.x - 1) / block.x,
-              maxblocks[0]),
+              m_max_grid_size[0]),
           std::min<array_index_type>(
               (m_rp.m_upper[1] - m_rp.m_lower[1] + block.y - 1) / block.y,
-              maxblocks[1]),
+              m_max_grid_size[1]),
           std::min<array_index_type>(
               (m_rp.m_upper[2] - m_rp.m_lower[2] + block.z - 1) / block.z,
-              maxblocks[2]));
+              m_max_grid_size[2]));
       // ensure we don't exceed the capability of the device
       check_grid_sizes(grid);
+
       CudaParallelLaunch<ParallelFor, LaunchBounds>(
           *this, grid, block, 0, m_rp.space().impl_internal_space_instance());
     } else if (RP::rank == 4) {
@@ -177,13 +181,13 @@ class ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>, Kokkos::Cuda> {
       check_block_sizes(block);
       const dim3 grid(
           std::min<array_index_type>(m_rp.m_tile_end[0] * m_rp.m_tile_end[1],
-                                     maxblocks[0]),
+                                     m_max_grid_size[0]),
           std::min<array_index_type>(
               (m_rp.m_upper[2] - m_rp.m_lower[2] + block.y - 1) / block.y,
-              maxblocks[1]),
+              m_max_grid_size[1]),
           std::min<array_index_type>(
               (m_rp.m_upper[3] - m_rp.m_lower[3] + block.z - 1) / block.z,
-              maxblocks[2]));
+              m_max_grid_size[2]));
       check_grid_sizes(grid);
       CudaParallelLaunch<ParallelFor, LaunchBounds>(
           *this, grid, block, 0, m_rp.space().impl_internal_space_instance());
@@ -195,12 +199,12 @@ class ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>, Kokkos::Cuda> {
       check_block_sizes(block);
       const dim3 grid(
           std::min<array_index_type>(m_rp.m_tile_end[0] * m_rp.m_tile_end[1],
-                                     maxblocks[0]),
+                                     m_max_grid_size[0]),
           std::min<array_index_type>(m_rp.m_tile_end[2] * m_rp.m_tile_end[3],
-                                     maxblocks[1]),
+                                     m_max_grid_size[1]),
           std::min<array_index_type>(
               (m_rp.m_upper[4] - m_rp.m_lower[4] + block.z - 1) / block.z,
-              maxblocks[2]));
+              m_max_grid_size[2]));
       check_grid_sizes(grid);
       CudaParallelLaunch<ParallelFor, LaunchBounds>(
           *this, grid, block, 0, m_rp.space().impl_internal_space_instance());
@@ -213,11 +217,11 @@ class ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>, Kokkos::Cuda> {
       check_block_sizes(block);
       const dim3 grid(
           std::min<array_index_type>(m_rp.m_tile_end[0] * m_rp.m_tile_end[1],
-                                     maxblocks[0]),
+                                     m_max_grid_size[0]),
           std::min<array_index_type>(m_rp.m_tile_end[2] * m_rp.m_tile_end[3],
-                                     maxblocks[1]),
+                                     m_max_grid_size[1]),
           std::min<array_index_type>(m_rp.m_tile_end[4] * m_rp.m_tile_end[5],
-                                     maxblocks[2]));
+                                     m_max_grid_size[2]));
       check_grid_sizes(grid);
       CudaParallelLaunch<ParallelFor, LaunchBounds>(
           *this, grid, block, 0, m_rp.space().impl_internal_space_instance());
@@ -229,7 +233,16 @@ class ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>, Kokkos::Cuda> {
 
   //  inline
   ParallelFor(const FunctorType& arg_functor, Policy arg_policy)
-      : m_functor(arg_functor), m_rp(arg_policy) {}
+      : m_functor(arg_functor),
+        m_rp(arg_policy),
+        m_max_grid_size({
+            static_cast<index_type>(
+                m_rp.space().cuda_device_prop().maxGridSize[0]),
+            static_cast<index_type>(
+                m_rp.space().cuda_device_prop().maxGridSize[1]),
+            static_cast<index_type>(
+                m_rp.space().cuda_device_prop().maxGridSize[2]),
+        }) {}
 };
 
 template <class CombinedFunctorReducerType, class... Traits>
@@ -378,7 +391,8 @@ class ParallelReduce<CombinedFunctorReducerType,
         Impl::ParallelReduce<CombinedFunctorReducer<FunctorType, ReducerType>,
                              Policy, Kokkos::Cuda>;
     cudaFuncAttributes attr = CudaParallelLaunch<closure_type, LaunchBounds>::
-        get_cuda_func_attributes(m_policy.space().cuda_device());
+        get_cuda_func_attributes(
+            m_policy.space().impl_internal_space_instance());
     while (
         (n && (maxShmemPerBlock < shmem_size)) ||
         (n >
