@@ -22,6 +22,7 @@
 
 #include <Kokkos_Core.hpp>
 #include <HIP/Kokkos_HIP_Space.hpp>
+#include <HIP/Kokkos_HIP_IsXnack.hpp>
 
 #include <HIP/Kokkos_HIP_DeepCopy.hpp>
 
@@ -53,12 +54,19 @@ namespace Kokkos {
 
 HIPSpace::HIPSpace()
     : m_device(HIP().hip_device()), m_stream(HIP().hip_stream()) {}
+HIPSpace::HIPSpace(int device_id, hipStream_t stream)
+    : m_device(device_id), m_stream(stream) {}
 
-HIPHostPinnedSpace::HIPHostPinnedSpace() {}
+HIPHostPinnedSpace::HIPHostPinnedSpace()
+    : m_device(HIP().hip_device()), m_stream(HIP().hip_stream()) {}
+HIPHostPinnedSpace::HIPHostPinnedSpace(int device_id, hipStream_t stream)
+    : m_device(device_id), m_stream(stream) {}
 
-HIPManagedSpace::HIPManagedSpace() : m_device(HIP().hip_device()) {}
+HIPManagedSpace::HIPManagedSpace()
+    : m_device(HIP().hip_device()), m_stream(HIP().hip_stream()) {}
+HIPManagedSpace::HIPManagedSpace(int device_id, hipStream_t stream)
+    : m_device(device_id), m_stream(stream) {}
 
-#ifndef KOKKOS_IMPL_HIP_UNIFIED_MEMORY
 void* HIPSpace::allocate(const HIP& exec_space,
                          const size_t arg_alloc_size) const {
   return allocate(exec_space, "[unlabeled]", arg_alloc_size);
@@ -67,10 +75,9 @@ void* HIPSpace::allocate(const HIP& exec_space,
 void* HIPSpace::allocate(const HIP& exec_space, const char* arg_label,
                          const size_t arg_alloc_size,
                          const size_t arg_logical_size) const {
-  return impl_allocate(exec_space.hip_stream(), arg_label, arg_alloc_size,
-                       arg_logical_size, true);
+  return impl_allocate(exec_space.hip_device(), exec_space.hip_stream(),
+                       arg_label, arg_alloc_size, arg_logical_size, true);
 }
-#endif
 
 void* HIPSpace::allocate(const size_t arg_alloc_size) const {
   return allocate("[unlabeled]", arg_alloc_size);
@@ -78,15 +85,23 @@ void* HIPSpace::allocate(const size_t arg_alloc_size) const {
 
 void* HIPSpace::allocate(const char* arg_label, const size_t arg_alloc_size,
                          const size_t arg_logical_size) const {
-  return impl_allocate(m_stream, arg_label, arg_alloc_size, arg_logical_size,
-                       false);
+  return impl_allocate(m_device, m_stream, arg_label, arg_alloc_size,
+                       arg_logical_size, false);
 }
 
-void* HIPSpace::impl_allocate(
-    [[maybe_unused]] const hipStream_t stream, const char* arg_label,
-    const size_t arg_alloc_size, const size_t arg_logical_size,
-    [[maybe_unused]] const bool stream_sync_only) const {
+void* HIPSpace::impl_allocate(const int device_id,
+                              [[maybe_unused]] const hipStream_t stream,
+                              const char* arg_label,
+                              const size_t arg_alloc_size,
+                              const size_t arg_logical_size,
+                              [[maybe_unused]] bool stream_sync_only) const {
   void* ptr = nullptr;
+  // ROCm 5.5 and earlier throw an error when using hipMallocAsync and
+  // arg_alloc_size is zero. Instead of trying to allocate memory, just return
+  // early.
+  if (arg_alloc_size == 0) return ptr;
+
+  KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(device_id));
 
 #ifdef KOKKOS_ENABLE_IMPL_HIP_MALLOC_ASYNC
   auto const error_code = hipMallocAsync(&ptr, arg_alloc_size, stream);
@@ -106,8 +121,7 @@ void* HIPSpace::impl_allocate(
     Kokkos::Impl::throw_bad_alloc(name(), arg_alloc_size, arg_label);
   }
   if (Kokkos::Profiling::profileLibraryLoaded()) {
-    const Kokkos::Tools::SpaceHandle arg_handle =
-        Kokkos::Tools::make_space_handle(name());
+    const auto arg_handle = Kokkos::Tools::make_space_handle(name());
     const size_t reported_size =
         (arg_logical_size > 0) ? arg_logical_size : arg_alloc_size;
     Kokkos::Profiling::allocateData(arg_handle, arg_label, ptr, reported_size);
@@ -130,6 +144,7 @@ void* HIPHostPinnedSpace::impl_allocate(
     const Kokkos::Tools::SpaceHandle arg_handle) const {
   void* ptr = nullptr;
 
+  KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(m_device));
   auto const error_code =
       hipHostMalloc(&ptr, arg_alloc_size, hipHostMallocNonCoherent);
   if (error_code != hipSuccess) {
@@ -162,6 +177,7 @@ void* HIPManagedSpace::impl_allocate(
   void* ptr = nullptr;
 
   if (arg_alloc_size > 0) {
+    KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(m_device));
     if (is_first_hip_managed_allocation.exchange(false) &&
         Kokkos::show_warnings()) {
       do {  // hack to avoid spamming users with too many warnings
@@ -176,23 +192,14 @@ Kokkos::HIP::allocation WARNING: The combination of device and system configurat
         }
 
         // check for correct runtime environment
-        const char* hsa_xnack = std::getenv("HSA_XNACK");
-        if (!hsa_xnack)
+        if (!Kokkos::Impl::xnack_environment_enabled())
           std::cerr << R"warning(
-Kokkos::HIP::runtime WARNING: Kokkos did not find an environment variable 'HSA_XNACK'
-                              for the current process.
-                              Nevertheless, xnack is enabled for all processes if
-                              amdgpu.noretry=0 was set in the Linux kernel boot line.
-                              Without xnack enabled, Kokkos::HIPManaged might not behave
-                              as expected.)warning"
-                    << std::endl;
-        else if (Kokkos::Impl::strcmp(hsa_xnack, "1") != 0)
-          std::cerr
-              << "Kokkos::HIP::runtime WARNING: Kokkos detected the "
-                 "environement variable "
-              << "'HSA_XNACK'=" << hsa_xnack << "\n"
-              << "Kokkos advises to set it to '1' to enable it per process."
-              << std::endl;
+Kokkos::HIP::runtime WARNING: Kokkos was not able to verify that xnack is enabled.
+                              Without xnack enabled, Kokkos::HIPManaged might not behave as expected.
+                              Set HSA_XNACK=1 in your environment. For further information on HMM support
+                              call `Kokkos::print_configuration`, or run with KOKKOS_PRINT_CONFIGURATION=1
+                              in your environment.
+)warning";
       } while (false);
     }
     auto const error_code = hipMallocManaged(&ptr, arg_alloc_size);
@@ -248,9 +255,11 @@ void HIPSpace::impl_deallocate(
                                       reported_size);
   }
 #ifdef KOKKOS_ENABLE_IMPL_HIP_MALLOC_ASYNC
+  KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(m_device));
   KOKKOS_IMPL_HIP_SAFE_CALL(hipFreeAsync(arg_alloc_ptr, m_stream));
   KOKKOS_IMPL_HIP_SAFE_CALL(hipDeviceSynchronize());
 #else
+  KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(m_device));
   KOKKOS_IMPL_HIP_SAFE_CALL(hipFree(arg_alloc_ptr));
 #endif
 }
@@ -276,6 +285,7 @@ void HIPHostPinnedSpace::impl_deallocate(
     Kokkos::Profiling::deallocateData(arg_handle, arg_label, arg_alloc_ptr,
                                       reported_size);
   }
+  KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(m_device));
   KOKKOS_IMPL_HIP_SAFE_CALL(hipHostFree(arg_alloc_ptr));
 }
 
@@ -305,6 +315,7 @@ void HIPManagedSpace::impl_deallocate(
   // kernel mem page table.
   KOKKOS_IMPL_HIP_SAFE_CALL(hipMemAdvise(
       arg_alloc_ptr, arg_alloc_size, hipMemAdviseUnsetCoarseGrain, m_device));
+  KOKKOS_IMPL_HIP_SAFE_CALL(hipSetDevice(m_device));
   KOKKOS_IMPL_HIP_SAFE_CALL(hipFree(arg_alloc_ptr));
 }
 
