@@ -30,6 +30,7 @@ static_assert(false,
 #include <impl/KokkosExp_Host_IterateTile.hpp>
 #include <Kokkos_ExecPolicy.hpp>
 #include <type_traits>
+#include <cmath>
 
 namespace Kokkos {
 
@@ -60,13 +61,13 @@ namespace Impl {
 // NOTE the comparison below is encapsulated to silent warnings about pointless
 // comparison of unsigned integer with zero
 template <class T>
-constexpr std::enable_if_t<!std::is_signed<T>::value, bool>
+constexpr std::enable_if_t<!std::is_signed_v<T>, bool>
 is_less_than_value_initialized_variable(T) {
   return false;
 }
 
 template <class T>
-constexpr std::enable_if_t<std::is_signed<T>::value, bool>
+constexpr std::enable_if_t<std::is_signed_v<T>, bool>
 is_less_than_value_initialized_variable(T arg) {
   return arg < T{};
 }
@@ -75,8 +76,8 @@ is_less_than_value_initialized_variable(T arg) {
 template <class To, class From>
 constexpr To checked_narrow_cast(From arg, std::size_t idx) {
   constexpr const bool is_different_signedness =
-      (std::is_signed<To>::value != std::is_signed<From>::value);
-  auto const ret = static_cast<To>(arg);
+      (std::is_signed_v<To> != std::is_signed_v<From>);
+  auto const ret = static_cast<To>(arg);  // NOLINT(bugprone-signed-char-misuse)
   if (static_cast<From>(ret) != arg ||
       (is_different_signedness &&
        is_less_than_value_initialized_variable(arg) !=
@@ -183,7 +184,7 @@ struct MDRangePolicy<P, Properties...>
   template <class... OtherProperties>
   friend struct MDRangePolicy;
 
-  static_assert(!std::is_void<typename traits::iteration_pattern>::value,
+  static_assert(!std::is_void_v<typename traits::iteration_pattern>,
                 "Kokkos Error: MD iteration pattern not defined");
 
   using iteration_pattern = typename traits::iteration_pattern;
@@ -238,9 +239,9 @@ struct MDRangePolicy<P, Properties...>
 
   template <typename LT, std::size_t LN, typename UT, std::size_t UN,
             typename TT = array_index_type, std::size_t TN = rank,
-            typename = std::enable_if_t<std::is_integral<LT>::value &&
-                                        std::is_integral<UT>::value &&
-                                        std::is_integral<TT>::value>>
+            typename = std::enable_if_t<std::is_integral_v<LT> &&
+                                        std::is_integral_v<UT> &&
+                                        std::is_integral_v<TT>>>
   MDRangePolicy(const LT (&lower)[LN], const UT (&upper)[UN],
                 const TT (&tile)[TN] = {})
       : MDRangePolicy(
@@ -257,9 +258,9 @@ struct MDRangePolicy<P, Properties...>
 
   template <typename LT, std::size_t LN, typename UT, std::size_t UN,
             typename TT = array_index_type, std::size_t TN = rank,
-            typename = std::enable_if_t<std::is_integral<LT>::value &&
-                                        std::is_integral<UT>::value &&
-                                        std::is_integral<TT>::value>>
+            typename = std::enable_if_t<std::is_integral_v<LT> &&
+                                        std::is_integral_v<UT> &&
+                                        std::is_integral_v<TT>>>
   MDRangePolicy(const typename traits::execution_space& work_space,
                 const LT (&lower)[LN], const UT (&upper)[UN],
                 const TT (&tile)[TN] = {})
@@ -291,14 +292,14 @@ struct MDRangePolicy<P, Properties...>
   }
 
   template <typename T, std::size_t NT = rank,
-            typename = std::enable_if_t<std::is_integral<T>::value>>
+            typename = std::enable_if_t<std::is_integral_v<T>>>
   MDRangePolicy(Kokkos::Array<T, rank> const& lower,
                 Kokkos::Array<T, rank> const& upper,
                 Kokkos::Array<T, NT> const& tile = Kokkos::Array<T, NT>{})
       : MDRangePolicy(typename traits::execution_space(), lower, upper, tile) {}
 
   template <typename T, std::size_t NT = rank,
-            typename = std::enable_if_t<std::is_integral<T>::value>>
+            typename = std::enable_if_t<std::is_integral_v<T>>>
   MDRangePolicy(const typename traits::execution_space& work_space,
                 Kokkos::Array<T, rank> const& lower,
                 Kokkos::Array<T, rank> const& upper,
@@ -330,7 +331,44 @@ struct MDRangePolicy<P, Properties...>
   }
   bool impl_tune_tile_size() const { return m_tune_tile_size; }
 
+  tile_type tile_size_recommended() const {
+    tile_type rec_tile_sizes = {};
+
+    for (std::size_t i = 0; i < rec_tile_sizes.size(); ++i) {
+      rec_tile_sizes[i] = tile_size_recommended(i);
+    }
+    return rec_tile_sizes;
+  }
+
+  int max_total_tile_size() const {
+    return Impl::get_tile_size_properties(m_space).max_total_tile_size;
+  }
+
  private:
+  int tile_size_recommended(const int tile_rank) const {
+    auto properties = Impl::get_tile_size_properties(m_space);
+    int last_rank   = (inner_direction == Iterate::Right) ? rank - 1 : 0;
+    int rank_acc =
+        (inner_direction == Iterate::Right) ? tile_rank + 1 : tile_rank - 1;
+    int rec_tile_size = (std::pow(properties.default_tile_size, rank_acc) <
+                         properties.max_total_tile_size)
+                            ? properties.default_tile_size
+                            : 1;
+
+    if (tile_rank == last_rank) {
+      rec_tile_size = tile_size_last_rank(
+          properties, m_upper[last_rank] - m_lower[last_rank]);
+    }
+    return rec_tile_size;
+  }
+
+  int tile_size_last_rank(const Impl::TileSizeProperties properties,
+                          const index_type length) const {
+    return properties.default_largest_tile_size == 0
+               ? std::max<int>(length, 1)
+               : properties.default_largest_tile_size;
+  }
+
   void init_helper(Impl::TileSizeProperties properties) {
     m_prod_tile_dims = 1;
     int increment    = 1;
@@ -341,6 +379,7 @@ struct MDRangePolicy<P, Properties...>
       rank_start = rank - 1;
       rank_end   = -1;
     }
+
     for (int i = rank_start; i != rank_end; i += increment) {
       const index_type length = m_upper[i] - m_lower[i];
 
@@ -368,9 +407,7 @@ struct MDRangePolicy<P, Properties...>
             m_tile[i] = 1;
           }
         } else {
-          m_tile[i] = properties.default_largest_tile_size == 0
-                          ? std::max<int>(length, 1)
-                          : properties.default_largest_tile_size;
+          m_tile[i] = tile_size_last_rank(properties, length);
         }
       }
       m_tile_end[i] =
@@ -389,58 +426,55 @@ struct MDRangePolicy<P, Properties...>
 };
 
 template <typename LT, size_t N, typename UT>
-MDRangePolicy(const LT (&)[N], const UT (&)[N])->MDRangePolicy<Rank<N>>;
+MDRangePolicy(const LT (&)[N], const UT (&)[N]) -> MDRangePolicy<Rank<N>>;
 
 template <typename LT, size_t N, typename UT, typename TT, size_t TN>
 MDRangePolicy(const LT (&)[N], const UT (&)[N], const TT (&)[TN])
-    ->MDRangePolicy<Rank<N>>;
+    -> MDRangePolicy<Rank<N>>;
 
 template <typename LT, size_t N, typename UT>
 MDRangePolicy(DefaultExecutionSpace const&, const LT (&)[N], const UT (&)[N])
-    ->MDRangePolicy<Rank<N>>;
+    -> MDRangePolicy<Rank<N>>;
 
 template <typename LT, size_t N, typename UT, typename TT, size_t TN>
 MDRangePolicy(DefaultExecutionSpace const&, const LT (&)[N], const UT (&)[N],
-              const TT (&)[TN])
-    ->MDRangePolicy<Rank<N>>;
+              const TT (&)[TN]) -> MDRangePolicy<Rank<N>>;
 
 template <typename ES, typename LT, size_t N, typename UT,
           typename = std::enable_if_t<is_execution_space_v<ES>>>
 MDRangePolicy(ES const&, const LT (&)[N], const UT (&)[N])
-    ->MDRangePolicy<ES, Rank<N>>;
+    -> MDRangePolicy<ES, Rank<N>>;
 
 template <typename ES, typename LT, size_t N, typename UT, typename TT,
           size_t TN, typename = std::enable_if_t<is_execution_space_v<ES>>>
 MDRangePolicy(ES const&, const LT (&)[N], const UT (&)[N], const TT (&)[TN])
-    ->MDRangePolicy<ES, Rank<N>>;
+    -> MDRangePolicy<ES, Rank<N>>;
 
 template <typename T, size_t N>
-MDRangePolicy(Array<T, N> const&, Array<T, N> const&)->MDRangePolicy<Rank<N>>;
+MDRangePolicy(Array<T, N> const&, Array<T, N> const&) -> MDRangePolicy<Rank<N>>;
 
 template <typename T, size_t N, size_t NT>
 MDRangePolicy(Array<T, N> const&, Array<T, N> const&, Array<T, NT> const&)
-    ->MDRangePolicy<Rank<N>>;
+    -> MDRangePolicy<Rank<N>>;
 
 template <typename T, size_t N>
 MDRangePolicy(DefaultExecutionSpace const&, Array<T, N> const&,
-              Array<T, N> const&)
-    ->MDRangePolicy<Rank<N>>;
+              Array<T, N> const&) -> MDRangePolicy<Rank<N>>;
 
 template <typename T, size_t N, size_t NT>
 MDRangePolicy(DefaultExecutionSpace const&, Array<T, N> const&,
               Array<T, N> const&, Array<T, NT> const&)
-    ->MDRangePolicy<Rank<N>>;
+    -> MDRangePolicy<Rank<N>>;
 
 template <typename ES, typename T, size_t N,
           typename = std::enable_if_t<is_execution_space_v<ES>>>
 MDRangePolicy(ES const&, Array<T, N> const&, Array<T, N> const&)
-    ->MDRangePolicy<ES, Rank<N>>;
+    -> MDRangePolicy<ES, Rank<N>>;
 
 template <typename ES, typename T, size_t N, size_t NT,
           typename = std::enable_if_t<is_execution_space_v<ES>>>
 MDRangePolicy(ES const&, Array<T, N> const&, Array<T, N> const&,
-              Array<T, NT> const&)
-    ->MDRangePolicy<ES, Rank<N>>;
+              Array<T, NT> const&) -> MDRangePolicy<ES, Rank<N>>;
 
 }  // namespace Kokkos
 
