@@ -100,18 +100,13 @@ inline constexpr bool is_standard_unsigned_integer_type_v =
 namespace Kokkos {
 
 //<editor-fold desc="[bit.cast], bit_cast">
-#if defined(KOKKOS_ENABLE_SYCL) && defined(__INTEL_LLVM_COMPILER) && \
-    __INTEL_LLVM_COMPILER < 20240000
-using sycl::detail::bit_cast;
-#else
 template <class To, class From>
 KOKKOS_FUNCTION std::enable_if_t<sizeof(To) == sizeof(From) &&
                                      std::is_trivially_copyable_v<To> &&
                                      std::is_trivially_copyable_v<From>,
                                  To>
 bit_cast(From const& from) noexcept {
-#if defined(KOKKOS_ENABLE_SYCL) && defined(__INTEL_LLVM_COMPILER) && \
-    __INTEL_LLVM_COMPILER >= 20240000
+#if defined(KOKKOS_ENABLE_SYCL)
   return sycl::bit_cast<To>(from);
 #else
   To to;
@@ -119,7 +114,6 @@ bit_cast(From const& from) noexcept {
   return to;
 #endif
 }
-#endif
 //</editor-fold>
 
 //<editor-fold desc="[bit.byteswap], byteswap">
@@ -411,6 +405,10 @@ KOKKOS_FUNCTION std::enable_if_t<std::is_integral_v<T>, T> byteswap_builtin(
     T x) noexcept {
   KOKKOS_IF_ON_DEVICE((return ::Kokkos::Impl::byteswap_builtin_device(x);))
   KOKKOS_IF_ON_HOST((return ::Kokkos::Impl::byteswap_builtin_host(x);))
+// FIXME-NVHPC: erroneous warning about return from non-void function
+#if defined(KOKKOS_ENABLE_OPENACC) && defined(KOKKOS_COMPILER_NVHPC)
+  return T();
+#endif
 }
 
 template <class T>
@@ -419,6 +417,10 @@ KOKKOS_FUNCTION std::enable_if_t<
 countl_zero_builtin(T x) noexcept {
   KOKKOS_IF_ON_DEVICE((return ::Kokkos::Impl::countl_zero_builtin_device(x);))
   KOKKOS_IF_ON_HOST((return ::Kokkos::Impl::countl_zero_builtin_host(x);))
+// FIXME-NVHPC: erroneous warning about return from non-void function
+#if defined(KOKKOS_ENABLE_OPENACC) && defined(KOKKOS_COMPILER_NVHPC)
+  return 0;
+#endif
 }
 
 template <class T>
@@ -435,6 +437,10 @@ KOKKOS_FUNCTION std::enable_if_t<
 countr_zero_builtin(T x) noexcept {
   KOKKOS_IF_ON_DEVICE((return ::Kokkos::Impl::countr_zero_builtin_device(x);))
   KOKKOS_IF_ON_HOST((return ::Kokkos::Impl::countr_zero_builtin_host(x);))
+// FIXME-NVHPC: erroneous warning about return from non-void function
+#if defined(KOKKOS_ENABLE_OPENACC) && defined(KOKKOS_COMPILER_NVHPC)
+  return 0;
+#endif
 }
 
 template <class T>
@@ -451,6 +457,10 @@ KOKKOS_FUNCTION std::enable_if_t<
 popcount_builtin(T x) noexcept {
   KOKKOS_IF_ON_DEVICE((return ::Kokkos::Impl::popcount_builtin_device(x);))
   KOKKOS_IF_ON_HOST((return ::Kokkos::Impl::popcount_builtin_host(x);))
+// FIXME-NVHPC: erroneous warning about return from non-void function
+#if defined(KOKKOS_ENABLE_OPENACC) && defined(KOKKOS_COMPILER_NVHPC)
+  return 0;
+#endif
 }
 
 template <class T>
