@@ -25,33 +25,40 @@ static_assert(false,
 #include <cstddef>
 #include <type_traits>
 #include <Kokkos_Macros.hpp>
+#ifdef KOKKOS_ENABLE_IMPL_MDSPAN
+#include <mdspan/mdspan.hpp>
+#else
+#include <limits>
+#endif
 
 namespace Kokkos {
+
+#ifndef KOKKOS_ENABLE_IMPL_MDSPAN
+constexpr size_t dynamic_extent = std::numeric_limits<size_t>::max();
+#endif
+
 namespace Experimental {
 
-constexpr ptrdiff_t dynamic_extent = -1;
-
-template <ptrdiff_t... ExtentSpecs>
+template <size_t... ExtentSpecs>
 struct Extents {
   /* TODO @enhancement flesh this out more */
 };
 
-template <class Exts, ptrdiff_t NewExtent>
+template <class Exts, size_t NewExtent>
 struct PrependExtent;
 
-template <ptrdiff_t... Exts, ptrdiff_t NewExtent>
+template <size_t... Exts, size_t NewExtent>
 struct PrependExtent<Extents<Exts...>, NewExtent> {
   using type = Extents<NewExtent, Exts...>;
 };
 
-template <class Exts, ptrdiff_t NewExtent>
+template <class Exts, size_t NewExtent>
 struct AppendExtent;
 
-template <ptrdiff_t... Exts, ptrdiff_t NewExtent>
+template <size_t... Exts, size_t NewExtent>
 struct AppendExtent<Extents<Exts...>, NewExtent> {
   using type = Extents<Exts..., NewExtent>;
 };
-
 }  // end namespace Experimental
 
 namespace Impl {
@@ -75,33 +82,32 @@ struct _parse_impl {
 
 // We have to treat the case of int**[x] specially, since it *doesn't* go
 // backwards
-template <class T, ptrdiff_t... ExtentSpec>
+template <class T, size_t... ExtentSpec>
 struct _parse_impl<T*, Kokkos::Experimental::Extents<ExtentSpec...>,
                    std::enable_if_t<_all_remaining_extents_dynamic<T>::value>>
-    : _parse_impl<T, Kokkos::Experimental::Extents<
-                         Kokkos::Experimental::dynamic_extent, ExtentSpec...>> {
-};
+    : _parse_impl<T, Kokkos::Experimental::Extents<Kokkos::dynamic_extent,
+                                                   ExtentSpec...>> {};
 
 // int*(*[x])[y] should still work also (meaning int[][x][][y])
-template <class T, ptrdiff_t... ExtentSpec>
+template <class T, size_t... ExtentSpec>
 struct _parse_impl<
     T*, Kokkos::Experimental::Extents<ExtentSpec...>,
     std::enable_if_t<!_all_remaining_extents_dynamic<T>::value>> {
   using _next = Kokkos::Experimental::AppendExtent<
       typename _parse_impl<T, Kokkos::Experimental::Extents<ExtentSpec...>,
                            void>::type,
-      Kokkos::Experimental::dynamic_extent>;
+      Kokkos::dynamic_extent>;
   using type = typename _next::type;
 };
 
-template <class T, ptrdiff_t... ExtentSpec, unsigned N>
+template <class T, size_t... ExtentSpec, unsigned N>
 struct _parse_impl<T[N], Kokkos::Experimental::Extents<ExtentSpec...>, void>
-    : _parse_impl<
-          T, Kokkos::Experimental::Extents<ExtentSpec...,
-                                           ptrdiff_t(N)>  // TODO @pedantic this
-                                                          // could be a
-                                                          // narrowing cast
-          > {};
+    : _parse_impl<T,
+                  Kokkos::Experimental::Extents<ExtentSpec...,
+                                                size_t(N)>  // TODO @pedantic
+                                                            // this could be a
+                                                            // narrowing cast
+                  > {};
 
 }  // end namespace _parse_view_extents_impl
 
@@ -111,38 +117,34 @@ struct ParseViewExtents {
       DataType, Kokkos::Experimental::Extents<>>::type;
 };
 
-template <class ValueType, ptrdiff_t Ext>
+template <class ValueType, size_t Ext>
 struct ApplyExtent {
   using type = ValueType[Ext];
 };
 
 template <class ValueType>
-struct ApplyExtent<ValueType, Kokkos::Experimental::dynamic_extent> {
+struct ApplyExtent<ValueType, Kokkos::dynamic_extent> {
   using type = ValueType*;
 };
 
-template <class ValueType, unsigned N, ptrdiff_t Ext>
+template <class ValueType, unsigned N, size_t Ext>
 struct ApplyExtent<ValueType[N], Ext> {
   using type = typename ApplyExtent<ValueType, Ext>::type[N];
 };
 
-template <class ValueType, ptrdiff_t Ext>
+template <class ValueType, size_t Ext>
 struct ApplyExtent<ValueType*, Ext> {
-  using type = ValueType * [Ext];
+  using type = ValueType* [Ext];
 };
 
 template <class ValueType>
-struct ApplyExtent<ValueType*, Kokkos::Experimental::dynamic_extent> {
-  using type =
-      typename ApplyExtent<ValueType,
-                           Kokkos::Experimental::dynamic_extent>::type*;
+struct ApplyExtent<ValueType*, dynamic_extent> {
+  using type = typename ApplyExtent<ValueType, dynamic_extent>::type*;
 };
 
 template <class ValueType, unsigned N>
-struct ApplyExtent<ValueType[N], Kokkos::Experimental::dynamic_extent> {
-  using type =
-      typename ApplyExtent<ValueType,
-                           Kokkos::Experimental::dynamic_extent>::type[N];
+struct ApplyExtent<ValueType[N], dynamic_extent> {
+  using type = typename ApplyExtent<ValueType, dynamic_extent>::type[N];
 };
 
 }  // end namespace Impl
