@@ -67,16 +67,7 @@ void *Kokkos::Experimental::OpenACCSpace::impl_allocate(
   ptr = acc_malloc(arg_alloc_size);
 
   if (!ptr) {
-    size_t alignment = 1;  // OpenACC does not handle alignment
-    using Kokkos::Experimental::RawMemoryAllocationFailure;
-    auto failure_mode =
-        arg_alloc_size > 0
-            ? RawMemoryAllocationFailure::FailureMode::OutOfMemoryError
-            : RawMemoryAllocationFailure::FailureMode::InvalidAllocationSize;
-    auto alloc_mechanism =
-        RawMemoryAllocationFailure::AllocationMechanism::OpenACCMalloc;
-    throw RawMemoryAllocationFailure(arg_alloc_size, alignment, failure_mode,
-                                     alloc_mechanism);
+    Kokkos::Impl::throw_bad_alloc(name(), arg_alloc_size, arg_label);
   }
 
   if (Kokkos::Profiling::profileLibraryLoaded()) {
