@@ -23,6 +23,7 @@
 #include <impl/Kokkos_Traits.hpp>
 
 #include <Kokkos_Atomic.hpp>
+#include <Kokkos_BitManipulation.hpp>
 #include "Kokkos_OpenMPTarget_Abort.hpp"
 #include <OpenMPTarget/Kokkos_OpenMPTarget_Macros.hpp>
 
@@ -554,9 +555,9 @@ class TeamPolicyInternal<Kokkos::Experimental::OpenMPTarget, Properties...>
 
     if (concurrency == 0) concurrency = 1;
 
-    if (m_chunk_size > 0) {
-      if (!Impl::is_integral_power_of_two(m_chunk_size))
-        Kokkos::abort("TeamPolicy blocking granularity must be power of two");
+    if (m_chunk_size > 0 &&
+        !Kokkos::has_single_bit(static_cast<unsigned>(m_chunk_size))) {
+      Kokkos::abort("TeamPolicy blocking granularity must be power of two");
     }
 
     int new_chunk_size = 1;
@@ -699,14 +700,14 @@ struct TeamThreadRangeBoundariesStruct<iType, OpenMPTargetExecTeamMember> {
   using index_type = iType;
   const iType start;
   const iType end;
-  const OpenMPTargetExecTeamMember& team;
+  const OpenMPTargetExecTeamMember& member;
 
-  TeamThreadRangeBoundariesStruct(const OpenMPTargetExecTeamMember& thread_,
-                                  iType count)
-      : start(0), end(count), team(thread_) {}
-  TeamThreadRangeBoundariesStruct(const OpenMPTargetExecTeamMember& thread_,
-                                  iType begin_, iType end_)
-      : start(begin_), end(end_), team(thread_) {}
+  TeamThreadRangeBoundariesStruct(const OpenMPTargetExecTeamMember& arg_thread,
+                                  iType arg_count)
+      : start(0), end(arg_count), member(arg_thread) {}
+  TeamThreadRangeBoundariesStruct(const OpenMPTargetExecTeamMember& arg_thread,
+                                  iType arg_begin, iType arg_end)
+      : start(arg_begin), end(arg_end), member(arg_thread) {}
 };
 
 template <typename iType>
@@ -714,14 +715,15 @@ struct ThreadVectorRangeBoundariesStruct<iType, OpenMPTargetExecTeamMember> {
   using index_type = iType;
   const index_type start;
   const index_type end;
-  const OpenMPTargetExecTeamMember& team;
+  const OpenMPTargetExecTeamMember& member;
 
-  ThreadVectorRangeBoundariesStruct(const OpenMPTargetExecTeamMember& thread_,
-                                    index_type count)
-      : start(0), end(count), team(thread_) {}
-  ThreadVectorRangeBoundariesStruct(const OpenMPTargetExecTeamMember& thread_,
-                                    index_type begin_, index_type end_)
-      : start(begin_), end(end_), team(thread_) {}
+  ThreadVectorRangeBoundariesStruct(
+      const OpenMPTargetExecTeamMember& arg_thread, index_type arg_count)
+      : start(0), end(arg_count), member(arg_thread) {}
+  ThreadVectorRangeBoundariesStruct(
+      const OpenMPTargetExecTeamMember& arg_thread, index_type arg_begin,
+      index_type arg_end)
+      : start(arg_begin), end(arg_end), member(arg_thread) {}
 };
 
 template <typename iType>
@@ -729,14 +731,14 @@ struct TeamVectorRangeBoundariesStruct<iType, OpenMPTargetExecTeamMember> {
   using index_type = iType;
   const index_type start;
   const index_type end;
-  const OpenMPTargetExecTeamMember& team;
+  const OpenMPTargetExecTeamMember& member;
 
-  TeamVectorRangeBoundariesStruct(const OpenMPTargetExecTeamMember& thread_,
-                                  index_type count)
-      : start(0), end(count), team(thread_) {}
-  TeamVectorRangeBoundariesStruct(const OpenMPTargetExecTeamMember& thread_,
-                                  index_type begin_, index_type end_)
-      : start(begin_), end(end_), team(thread_) {}
+  TeamVectorRangeBoundariesStruct(const OpenMPTargetExecTeamMember& arg_thread,
+                                  index_type arg_count)
+      : start(0), end(arg_count), member(arg_thread) {}
+  TeamVectorRangeBoundariesStruct(const OpenMPTargetExecTeamMember& arg_thread,
+                                  index_type arg_begin, index_type arg_end)
+      : start(arg_begin), end(arg_end), member(arg_thread) {}
 };
 
 }  // namespace Impl
