@@ -17,6 +17,10 @@
 #ifndef KOKKOS_SYCL_PARALLEL_FOR_MDRANGE_HPP_
 #define KOKKOS_SYCL_PARALLEL_FOR_MDRANGE_HPP_
 
+#include <limits>
+
+#include <sycl/sycl.hpp>
+
 #include <impl/KokkosExp_IterateTileGPU.hpp>
 
 #ifdef KOKKOS_IMPL_SYCL_USE_IN_ORDER_QUEUES
@@ -33,6 +37,7 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
   using array_index_type = typename Policy::array_index_type;
   using index_type       = typename Policy::index_type;
   using WorkTag          = typename Policy::work_tag;
+  using MaxGridSize      = Kokkos::Array<index_type, 3>;
 
   const FunctorType m_functor;
   // MDRangePolicy is not trivially copyable. Hence, replicate the data we
@@ -54,6 +59,7 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
     const typename Policy::index_type m_num_tiles;
     static constexpr Iterate inner_direction = Policy::inner_direction;
   } m_policy;
+  const MaxGridSize m_max_grid_size;
   const Kokkos::SYCL& m_space;
 
   sycl::nd_range<3> compute_ranges() const {
@@ -61,25 +67,46 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
     const auto& m_tile_end = m_policy.m_tile_end;
 
     if constexpr (Policy::rank == 2) {
+      // id0 to threadIdx.x; id1 to threadIdx.y
       sycl::range<3> local_sizes(m_tile[0], m_tile[1], 1);
-      sycl::range<3> global_sizes(m_tile_end[0] * m_tile[0],
-                                  m_tile_end[1] * m_tile[1], 1);
+
+      sycl::range<3> global_sizes(
+          std::min<array_index_type>(m_tile_end[0], m_max_grid_size[0]) *
+              m_tile[0],
+          std::min<array_index_type>(m_tile_end[1], m_max_grid_size[1]) *
+              m_tile[1],
+          1);
+
       return {global_sizes, local_sizes};
     }
     if constexpr (Policy::rank == 3) {
+      // id0 to threadIdx.x; id1 to threadIdx.y; id2 to threadIdx.z
       sycl::range<3> local_sizes(m_tile[0], m_tile[1], m_tile[2]);
-      sycl::range<3> global_sizes(m_tile_end[0] * m_tile[0],
-                                  m_tile_end[1] * m_tile[1],
-                                  m_tile_end[2] * m_tile[2]);
+
+      sycl::range<3> global_sizes(
+          std::min<array_index_type>(m_tile_end[0], m_max_grid_size[0]) *
+              m_tile[0],
+          std::min<array_index_type>(m_tile_end[1], m_max_grid_size[1]) *
+              m_tile[1],
+          std::min<array_index_type>(m_tile_end[2], m_max_grid_size[2]) *
+              m_tile[2]);
+
       return {global_sizes, local_sizes};
     }
     if constexpr (Policy::rank == 4) {
       // id0,id1 encoded within first index; id2 to second index; id3 to third
       // index
       sycl::range<3> local_sizes(m_tile[0] * m_tile[1], m_tile[2], m_tile[3]);
+
       sycl::range<3> global_sizes(
-          m_tile_end[0] * m_tile[0] * m_tile_end[1] * m_tile[1],
-          m_tile_end[2] * m_tile[2], m_tile_end[3] * m_tile[3]);
+          std::min<array_index_type>(m_tile_end[0] * m_tile_end[1],
+                                     m_max_grid_size[0]) *
+              m_tile[0] * m_tile[1],
+          std::min<array_index_type>(m_tile_end[2], m_max_grid_size[1]) *
+              m_tile[2],
+          std::min<array_index_type>(m_tile_end[3], m_max_grid_size[2]) *
+              m_tile[3]);
+
       return {global_sizes, local_sizes};
     }
     if constexpr (Policy::rank == 5) {
@@ -87,10 +114,17 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
       // third index
       sycl::range<3> local_sizes(m_tile[0] * m_tile[1], m_tile[2] * m_tile[3],
                                  m_tile[4]);
+
       sycl::range<3> global_sizes(
-          m_tile_end[0] * m_tile[0] * m_tile_end[1] * m_tile[1],
-          m_tile_end[2] * m_tile[2] * m_tile_end[3] * m_tile[3],
-          m_tile_end[4] * m_tile[4]);
+          std::min<array_index_type>(m_tile_end[0] * m_tile_end[1],
+                                     m_max_grid_size[0]) *
+              m_tile[0] * m_tile[1],
+          std::min<array_index_type>(m_tile_end[2] * m_tile_end[3],
+                                     m_max_grid_size[1]) *
+              m_tile[2] * m_tile[3],
+          std::min<array_index_type>(m_tile_end[4], m_max_grid_size[2]) *
+              m_tile[4]);
+
       return {global_sizes, local_sizes};
     }
     if constexpr (Policy::rank == 6) {
@@ -98,10 +132,18 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
       // third index
       sycl::range<3> local_sizes(m_tile[0] * m_tile[1], m_tile[2] * m_tile[3],
                                  m_tile[4] * m_tile[5]);
+
       sycl::range<3> global_sizes(
-          m_tile_end[0] * m_tile[0] * m_tile_end[1] * m_tile[1],
-          m_tile_end[2] * m_tile[2] * m_tile_end[3] * m_tile[3],
-          m_tile_end[4] * m_tile[4] * m_tile_end[5] * m_tile[5]);
+          std::min<array_index_type>(m_tile_end[0] * m_tile_end[1],
+                                     m_max_grid_size[0]) *
+              m_tile[0] * m_tile[1],
+          std::min<array_index_type>(m_tile_end[2] * m_tile_end[3],
+                                     m_max_grid_size[1]) *
+              m_tile[2] * m_tile[3],
+          std::min<array_index_type>(m_tile_end[4] * m_tile_end[5],
+                                     m_max_grid_size[2]) *
+              m_tile[4] * m_tile[5]);
+
       return {global_sizes, local_sizes};
     }
     static_assert(Policy::rank > 1 && Policy::rank < 7,
@@ -117,6 +159,7 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
     if (m_policy.m_num_tiles == 0) return {};
 
     const BarePolicy bare_policy(m_policy);
+    const auto& max_grid_size = m_max_grid_size;
 
     desul::ensure_sycl_lock_arrays_on_device(q);
 
@@ -133,7 +176,8 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
 #else
       (void)memcpy_event;
 #endif
-      cgh.parallel_for(sycl_swapped_range, [functor_wrapper, bare_policy](
+      cgh.parallel_for(sycl_swapped_range, [functor_wrapper, bare_policy,
+                                            max_grid_size](
                                                sycl::nd_item<3> item) {
         // swap back for correct index calculations in DeviceIterateTile
         const index_type local_x    = item.get_local_id(2);
@@ -147,15 +191,15 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
         const index_type n_global_z = item.get_group_range(0);
 
         Kokkos::Impl::DeviceIterateTile<Policy::rank, BarePolicy, FunctorType,
-                                        typename Policy::work_tag>(
-            bare_policy, functor_wrapper.get_functor(),
+                                        MaxGridSize, typename Policy::work_tag>(
+            bare_policy, functor_wrapper.get_functor(), max_grid_size,
             {n_global_x, n_global_y, n_global_z},
             {global_x, global_y, global_z}, {local_x, local_y, local_z})
             .exec_range();
       });
     };
 
-#ifdef SYCL_EXT_ONEAPI_GRAPH
+#ifdef KOKKOS_IMPL_SYCL_GRAPH_SUPPORT
     if constexpr (Policy::is_graph_kernel::value) {
       sycl_attach_kernel_to_node(*this, cgh_lambda);
       return {};
@@ -171,6 +215,34 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
     }
   }
 
+  static MaxGridSize get_max_grid_size(const Policy& policy) {
+    // the SYCL specs do not allow to get the maximum grid size (maximum
+    // ND-range size, maximum number of work groups)
+    // TODO update this when the specs change
+#ifdef SYCL_EXT_ONEAPI_MAX_WORK_GROUP_QUERY
+    // we use an Intel extension if possible
+    auto max_grid_size =
+        policy.space()
+            .sycl_queue()
+            .get_device()
+            .template get_info<sycl::ext::oneapi::experimental::info::device::
+                                   max_work_groups<3>>();
+
+    // note that SYCL represents a (x, y, z) range with the the right-most term
+    // as the one varying the fastest, so the order must be reversed for Kokkos
+    // see:
+    // https://registry.khronos.org/SYCL/specs/sycl-2020/html/sycl-2020.html#sec:multi-dim-linearization
+    return {static_cast<index_type>(max_grid_size[2]),
+            static_cast<index_type>(max_grid_size[1]),
+            static_cast<index_type>(max_grid_size[0])};
+#else
+    // otherwise, we consider that the max is infinite
+    return {std::numeric_limits<index_type>::max(),
+            std::numeric_limits<index_type>::max(),
+            std::numeric_limits<index_type>::max()};
+#endif
+  }
+
  public:
   using functor_type = FunctorType;
 
@@ -193,6 +265,7 @@ class Kokkos::Impl::ParallelFor<FunctorType, Kokkos::MDRangePolicy<Traits...>,
   ParallelFor(const FunctorType& arg_functor, const Policy& arg_policy)
       : m_functor(arg_functor),
         m_policy(arg_policy),
+        m_max_grid_size(get_max_grid_size(arg_policy)),
         m_space(arg_policy.space()) {}
 };
 
