@@ -18,6 +18,7 @@
 #define KOKKOS_SYCL_PARALLEL_SCAN_RANGE_HPP
 
 #include <Kokkos_Macros.hpp>
+#include <SYCL/Kokkos_SYCL_WorkgroupReduction.hpp>
 #include <memory>
 #include <vector>
 
@@ -35,20 +36,38 @@ void workgroup_scan(sycl::nd_item<dim> item, const FunctorType& final_reducer,
   auto sg               = item.get_sub_group();
   const int sg_group_id = sg.get_group_id()[0];
   const int id_in_sg    = sg.get_local_id()[0];
+  const int local_range = std::min<int>(sg.get_local_range()[0], global_range);
 
-  for (int stride = 1; stride < global_range; stride <<= 1) {
-    auto tmp = sg.shuffle_up(local_value, stride);
+#if defined(KOKKOS_ARCH_INTEL_GPU) || defined(KOKKOS_IMPL_ARCH_NVIDIA_GPU)
+  auto shuffle_combine = [&](int stride) {
+    if (stride < local_range) {
+      auto tmp = Kokkos::Impl::SYCLReduction::shift_group_right(sg, local_value,
+                                                                stride);
+      if (id_in_sg >= stride) final_reducer.join(&local_value, &tmp);
+    }
+  };
+  shuffle_combine(1);
+  shuffle_combine(2);
+  shuffle_combine(4);
+  shuffle_combine(8);
+  shuffle_combine(16);
+  KOKKOS_ASSERT(local_range <= 32);
+#else
+  for (int stride = 1; stride < local_range; stride <<= 1) {
+    auto tmp =
+        Kokkos::Impl::SYCLReduction::shift_group_right(sg, local_value, stride);
     if (id_in_sg >= stride) final_reducer.join(&local_value, &tmp);
   }
+#endif
 
   const int max_subgroup_size = sg.get_max_local_range()[0];
   const int n_active_subgroups =
       (global_range + max_subgroup_size - 1) / max_subgroup_size;
 
-  const int local_range = sg.get_local_range()[0];
   if (id_in_sg == local_range - 1 && sg_group_id < n_active_subgroups)
     local_mem[sg_group_id] = local_value;
-  local_value = sg.shuffle_up(local_value, 1);
+  local_value =
+      Kokkos::Impl::SYCLReduction::shift_group_right(sg, local_value, 1);
   if (id_in_sg == 0) final_reducer.init(&local_value);
   sycl::group_barrier(item.get_group());
 
@@ -61,8 +80,29 @@ void workgroup_scan(sycl::nd_item<dim> item, const FunctorType& final_reducer,
         const auto upper_bound =
             std::min(local_range, n_active_subgroups - round * local_range);
         auto local_sg_value = local_mem[idx < n_active_subgroups ? idx : 0];
+#if defined(KOKKOS_ARCH_INTEL_GPU) || defined(KOKKOS_IMPL_ARCH_NVIDIA_GPU)
+        auto shuffle_combine_sg = [&](int stride) {
+          if (stride < upper_bound) {
+            auto tmp = Kokkos::Impl::SYCLReduction::shift_group_right(
+                sg, local_sg_value, stride);
+            if (id_in_sg >= stride) {
+              if (idx < n_active_subgroups)
+                final_reducer.join(&local_sg_value, &tmp);
+              else
+                local_sg_value = tmp;
+            }
+          }
+        };
+        shuffle_combine_sg(1);
+        shuffle_combine_sg(2);
+        shuffle_combine_sg(4);
+        shuffle_combine_sg(8);
+        shuffle_combine_sg(16);
+        KOKKOS_ASSERT(upper_bound <= 32);
+#else
         for (int stride = 1; stride < upper_bound; stride <<= 1) {
-          auto tmp = sg.shuffle_up(local_sg_value, stride);
+          auto tmp = Kokkos::Impl::SYCLReduction::shift_group_right(
+              sg, local_sg_value, stride);
           if (id_in_sg >= stride) {
             if (idx < n_active_subgroups)
               final_reducer.join(&local_sg_value, &tmp);
@@ -70,6 +110,7 @@ void workgroup_scan(sycl::nd_item<dim> item, const FunctorType& final_reducer,
               local_sg_value = tmp;
           }
         }
+#endif
         if (idx < n_active_subgroups) {
           local_mem[idx] = local_sg_value;
           if (round > 0)
@@ -104,122 +145,116 @@ class ParallelScanSYCLBase {
   using value_type     = typename Analysis::value_type;
   using reference_type = typename Analysis::reference_type;
   using functor_type   = FunctorType;
-  using size_type      = Kokkos::Experimental::SYCL::size_type;
+  using size_type      = Kokkos::SYCL::size_type;
   using index_type     = typename Policy::index_type;
 
  protected:
   const CombinedFunctorReducer<FunctorType, typename Analysis::Reducer>
       m_functor_reducer;
   const Policy m_policy;
-  sycl::host_ptr<value_type> m_scratch_host = nullptr;
+  sycl_host_ptr<value_type> m_scratch_host = nullptr;
   pointer_type m_result_ptr;
   const bool m_result_ptr_device_accessible;
 
-  // Only let one ParallelScan instance at a time use the host scratch memory.
-  // The constructor acquires the mutex which is released in the destructor.
-  std::scoped_lock<std::mutex> m_scratch_buffers_lock;
-
  private:
   template <typename FunctorWrapper>
   sycl::event sycl_direct_launch(const FunctorWrapper& functor_wrapper,
                                  sycl::event memcpy_event) {
     // Convenience references
-    const Kokkos::Experimental::SYCL& space = m_policy.space();
-    Kokkos::Experimental::Impl::SYCLInternal& instance =
+    const Kokkos::SYCL& space = m_policy.space();
+    Kokkos::Impl::SYCLInternal& instance =
         *space.impl_internal_space_instance();
     sycl::queue& q = space.sycl_queue();
 
     const auto size = m_policy.end() - m_policy.begin();
 
-    auto scratch_flags = static_cast<sycl::device_ptr<unsigned int>>(
+    auto scratch_flags = static_cast<sycl_device_ptr<unsigned int>>(
         instance.scratch_flags(sizeof(unsigned int)));
 
     const auto begin = m_policy.begin();
 
     // Initialize global memory
-    auto scan_lambda_factory =
-        [&](sycl::local_accessor<value_type> local_mem,
-            sycl::local_accessor<unsigned int> num_teams_done,
-            sycl::device_ptr<value_type> global_mem_,
-            sycl::device_ptr<value_type> group_results_) {
-          auto lambda = [=](sycl::nd_item<1> item) {
-            auto global_mem    = global_mem_;
-            auto group_results = group_results_;
+    auto scan_lambda_factory = [&](sycl::local_accessor<value_type> local_mem,
+                                   sycl::local_accessor<unsigned int>
+                                       num_teams_done,
+                                   sycl_device_ptr<value_type> global_mem_,
+                                   sycl_device_ptr<value_type> group_results_) {
+      auto lambda = [=](sycl::nd_item<1> item) {
+        auto global_mem    = global_mem_;
+        auto group_results = group_results_;
 
-            const CombinedFunctorReducer<
-                FunctorType, typename Analysis::Reducer>& functor_reducer =
-                functor_wrapper.get_functor();
-            const FunctorType& functor = functor_reducer.get_functor();
-            const typename Analysis::Reducer& reducer =
-                functor_reducer.get_reducer();
+        const CombinedFunctorReducer<FunctorType, typename Analysis::Reducer>&
+            functor_reducer        = functor_wrapper.get_functor();
+        const FunctorType& functor = functor_reducer.get_functor();
+        const typename Analysis::Reducer& reducer =
+            functor_reducer.get_reducer();
 
-            const auto n_wgroups  = item.get_group_range()[0];
-            const int wgroup_size = item.get_local_range()[0];
+        const auto n_wgroups  = item.get_group_range()[0];
+        const int wgroup_size = item.get_local_range()[0];
 
-            const int local_id         = item.get_local_linear_id();
-            const index_type global_id = item.get_global_linear_id();
+        const int local_id         = item.get_local_linear_id();
+        const index_type global_id = item.get_global_linear_id();
 
-            // Initialize local memory
-            value_type local_value;
-            reducer.init(&local_value);
-            if (global_id < size) {
-              if constexpr (std::is_void<WorkTag>::value)
-                functor(global_id + begin, local_value, false);
-              else
-                functor(WorkTag(), global_id + begin, local_value, false);
+        // Initialize local memory
+        value_type local_value;
+        reducer.init(&local_value);
+        if (global_id < size) {
+          if constexpr (std::is_void_v<WorkTag>)
+            functor(global_id + begin, local_value, false);
+          else
+            functor(WorkTag(), global_id + begin, local_value, false);
+        }
+
+        workgroup_scan<>(item, reducer, local_mem, local_value, wgroup_size);
+
+        // Write results to global memory
+        if (global_id < size) global_mem[global_id] = local_value;
+
+        if (local_id == wgroup_size - 1) {
+          group_results[item.get_group_linear_id()] =
+              local_mem[item.get_sub_group().get_group_range()[0] - 1];
+
+          sycl::atomic_ref<unsigned, sycl::memory_order::acq_rel,
+                           sycl::memory_scope::device,
+                           sycl::access::address_space::global_space>
+              scratch_flags_ref(*scratch_flags);
+          num_teams_done[0] = ++scratch_flags_ref;
+        }
+        sycl::group_barrier(item.get_group());
+        if (num_teams_done[0] == n_wgroups) {
+          if (local_id == 0) *scratch_flags = 0;
+          value_type total;
+          reducer.init(&total);
+
+          for (unsigned int offset = 0; offset < n_wgroups;
+               offset += wgroup_size) {
+            index_type id = local_id + offset;
+            if (id < static_cast<index_type>(n_wgroups))
+              local_value = group_results[id];
+            else
+              reducer.init(&local_value);
+            workgroup_scan<>(
+                item, reducer, local_mem, local_value,
+                std::min<index_type>(n_wgroups - offset, wgroup_size));
+            if (id < static_cast<index_type>(n_wgroups)) {
+              reducer.join(&local_value, &total);
+              group_results[id] = local_value;
             }
-
-            workgroup_scan<>(item, reducer, local_mem, local_value,
-                             wgroup_size);
-
-            // Write results to global memory
-            if (global_id < size) global_mem[global_id] = local_value;
-
-            if (local_id == wgroup_size - 1) {
-              group_results[item.get_group_linear_id()] =
-                  local_mem[item.get_sub_group().get_group_range()[0] - 1];
-
-              sycl::atomic_ref<unsigned, sycl::memory_order::acq_rel,
-                               sycl::memory_scope::device,
-                               sycl::access::address_space::global_space>
-                  scratch_flags_ref(*scratch_flags);
-              num_teams_done[0] = ++scratch_flags_ref;
-            }
-            item.barrier(sycl::access::fence_space::global_space);
-            if (num_teams_done[0] == n_wgroups) {
-              if (local_id == 0) *scratch_flags = 0;
-              value_type total;
-              reducer.init(&total);
-
-              for (unsigned int offset = 0; offset < n_wgroups;
-                   offset += wgroup_size) {
-                index_type id = local_id + offset;
-                if (id < static_cast<index_type>(n_wgroups))
-                  local_value = group_results[id];
-                else
-                  reducer.init(&local_value);
-                workgroup_scan<>(
-                    item, reducer, local_mem, local_value,
-                    std::min<index_type>(n_wgroups - offset, wgroup_size));
-                if (id < static_cast<index_type>(n_wgroups)) {
-                  reducer.join(&local_value, &total);
-                  group_results[id] = local_value;
-                }
-                reducer.join(
-                    &total,
-                    &local_mem[item.get_sub_group().get_group_range()[0] - 1]);
-                if (offset + wgroup_size < n_wgroups)
-                  item.barrier(sycl::access::fence_space::global_space);
-              }
-            }
-          };
-          return lambda;
-        };
+            reducer.join(
+                &total,
+                &local_mem[item.get_sub_group().get_group_range()[0] - 1]);
+            if (offset + wgroup_size < n_wgroups)
+              sycl::group_barrier(item.get_group());
+          }
+        }
+      };
+      return lambda;
+    };
 
     size_t wgroup_size;
     size_t n_wgroups;
-    sycl::device_ptr<value_type> global_mem;
-    sycl::device_ptr<value_type> group_results;
+    sycl_device_ptr<value_type> global_mem;
+    sycl_device_ptr<value_type> group_results;
 
     desul::ensure_sycl_lock_arrays_on_device(q);
 
@@ -254,9 +289,9 @@ class ParallelScanSYCLBase {
       // FIXME_SYCL consider only storing one value per block and recreate
       // initial results in the end before doing the final pass
       global_mem =
-          static_cast<sycl::device_ptr<value_type>>(instance.scratch_space(
+          static_cast<sycl_device_ptr<value_type>>(instance.scratch_space(
               n_wgroups * (wgroup_size + 1) * sizeof(value_type)));
-      m_scratch_host = static_cast<sycl::host_ptr<value_type>>(
+      m_scratch_host = static_cast<sycl_host_ptr<value_type>>(
           instance.scratch_host(sizeof(value_type)));
 
       group_results = global_mem + n_wgroups * wgroup_size;
@@ -311,7 +346,7 @@ class ParallelScanSYCLBase {
 
               reducer.join(&update, &group_results[item.get_group_linear_id()]);
 
-              if constexpr (std::is_void<WorkTag>::value)
+              if constexpr (std::is_void_v<WorkTag>)
                 functor(global_id + begin, update, true);
               else
                 functor(WorkTag(), global_id + begin, update, true);
@@ -334,11 +369,16 @@ class ParallelScanSYCLBase {
 
     auto& instance = *m_policy.space().impl_internal_space_instance();
 
-    Kokkos::Experimental::Impl::SYCLInternal::IndirectKernelMem&
-        indirectKernelMem = instance.get_indirect_kernel_mem();
+    // Only let one instance at a time resize the instance's scratch memory
+    // allocations.
+    std::scoped_lock<std::mutex> scratch_buffers_lock(
+        instance.m_mutexScratchSpace);
 
-    auto functor_wrapper = Experimental::Impl::make_sycl_function_wrapper(
-        m_functor_reducer, indirectKernelMem);
+    Kokkos::Impl::SYCLInternal::IndirectKernelMem& indirectKernelMem =
+        instance.get_indirect_kernel_mem();
+
+    auto functor_wrapper =
+        Impl::make_sycl_function_wrapper(m_functor_reducer, indirectKernelMem);
 
     sycl::event event =
         sycl_direct_launch(functor_wrapper, functor_wrapper.get_copy_event());
@@ -352,17 +392,14 @@ class ParallelScanSYCLBase {
       : m_functor_reducer(arg_functor, typename Analysis::Reducer{arg_functor}),
         m_policy(arg_policy),
         m_result_ptr(arg_result_ptr),
-        m_result_ptr_device_accessible(arg_result_ptr_device_accessible),
-        m_scratch_buffers_lock(m_policy.space()
-                                   .impl_internal_space_instance()
-                                   ->m_mutexScratchSpace) {}
+        m_result_ptr_device_accessible(arg_result_ptr_device_accessible) {}
 };
 
 }  // namespace Kokkos::Impl
 
 template <class FunctorType, class... Traits>
 class Kokkos::Impl::ParallelScan<FunctorType, Kokkos::RangePolicy<Traits...>,
-                                 Kokkos::Experimental::SYCL>
+                                 Kokkos::SYCL>
     : private ParallelScanSYCLBase<FunctorType, void, Traits...> {
  public:
   using Base = ParallelScanSYCLBase<FunctorType, void, Traits...>;
@@ -380,13 +417,12 @@ class Kokkos::Impl::ParallelScan<FunctorType, Kokkos::RangePolicy<Traits...>,
 
 template <class FunctorType, class ReturnType, class... Traits>
 class Kokkos::Impl::ParallelScanWithTotal<
-    FunctorType, Kokkos::RangePolicy<Traits...>, ReturnType,
-    Kokkos::Experimental::SYCL>
+    FunctorType, Kokkos::RangePolicy<Traits...>, ReturnType, Kokkos::SYCL>
     : public ParallelScanSYCLBase<FunctorType, ReturnType, Traits...> {
  public:
   using Base = ParallelScanSYCLBase<FunctorType, ReturnType, Traits...>;
 
-  const Kokkos::Experimental::SYCL& m_exec;
+  const Kokkos::SYCL& m_exec;
 
   inline void execute() {
     Base::impl_execute([&]() {
@@ -398,7 +434,8 @@ class Kokkos::Impl::ParallelScanWithTotal<
             "Kokkos::Impl::ParallelReduce<SYCL, MDRangePolicy>::execute: "
             "result not device-accessible");
         const int size = Base::m_functor_reducer.get_reducer().value_size();
-        std::memcpy(Base::m_result_ptr, Base::m_scratch_host, size);
+        std::memcpy(static_cast<void*>(Base::m_result_ptr),
+                    static_cast<const void*>(Base::m_scratch_host), size);
       }
     });
   }
@@ -408,7 +445,7 @@ class Kokkos::Impl::ParallelScanWithTotal<
                         const typename Base::Policy& arg_policy,
                         const ViewType& arg_result_view)
       : Base(arg_functor, arg_policy, arg_result_view.data(),
-             MemorySpaceAccess<Experimental::SYCLDeviceUSMSpace,
+             MemorySpaceAccess<SYCLDeviceUSMSpace,
                                typename ViewType::memory_space>::accessible),
         m_exec(arg_policy.space()) {}
 };
