@@ -25,6 +25,7 @@
 #include <impl/Kokkos_HostThreadTeam.hpp>
 
 #include <Kokkos_Atomic.hpp>
+#include <Kokkos_BitManipulation.hpp>
 #include <Threads/Kokkos_Threads_Spinwait.hpp>
 #include <Threads/Kokkos_Threads_State.hpp>
 
@@ -143,8 +144,8 @@ class ThreadsExecTeamMember {
 
     KOKKOS_IF_ON_HOST((
         // Make sure there is enough scratch space:
-        using type = typename if_c<sizeof(ValueType) < TEAM_REDUCE_SIZE,
-                                   ValueType, void>::type;
+        using type = std::conditional_t<sizeof(ValueType) < TEAM_REDUCE_SIZE,
+                                        ValueType, void>;
 
         if (m_team_base) {
           type* const local_value = ((type*)m_team_base[0]->scratch_memory());
@@ -164,8 +165,8 @@ class ThreadsExecTeamMember {
 
     KOKKOS_IF_ON_HOST((
         // Make sure there is enough scratch space:
-        using type = typename if_c<sizeof(ValueType) < TEAM_REDUCE_SIZE,
-                                   ValueType, void>::type;
+        using type = std::conditional_t<sizeof(ValueType) < TEAM_REDUCE_SIZE,
+                                        ValueType, void>;
         f(value); if (m_team_base) {
           type* const local_value = ((type*)m_team_base[0]->scratch_memory());
           memory_fence();
@@ -186,9 +187,7 @@ class ThreadsExecTeamMember {
     KOKKOS_IF_ON_HOST((
         // Make sure there is enough scratch space:
         using type =
-            typename if_c<sizeof(Type) < TEAM_REDUCE_SIZE, Type, void>::type;
-
-        if (m_instance == nullptr) return value;
+            std::conditional_t<sizeof(Type) < TEAM_REDUCE_SIZE, Type, void>;
 
         if (team_rank() != team_size() - 1) *
             ((volatile type*)m_instance->scratch_memory()) = value;
@@ -217,54 +216,65 @@ class ThreadsExecTeamMember {
   }
 
   template <typename ReducerType>
-  KOKKOS_INLINE_FUNCTION
-      std::enable_if_t<Kokkos::is_reducer<ReducerType>::value>
-      team_reduce(const ReducerType& reducer,
-                  const typename ReducerType::value_type contribution) const {
+  KOKKOS_INLINE_FUNCTION std::enable_if_t<is_reducer<ReducerType>::value>
+  team_reduce(const ReducerType& reducer,
+              typename ReducerType::value_type& contribution) const {
     KOKKOS_IF_ON_DEVICE(((void)reducer; (void)contribution;))
 
-    KOKKOS_IF_ON_HOST((
-        using value_type = typename ReducerType::value_type;
-        // Make sure there is enough scratch space:
-        using type = typename if_c<sizeof(value_type) < TEAM_REDUCE_SIZE,
-                                   value_type, void>::type;
+    KOKKOS_IF_ON_HOST(
+        (using value_type           = typename ReducerType::value_type;
+         using wrapped_reducer_type = typename Impl::FunctorAnalysis<
+             Impl::FunctorPatternInterface::REDUCE, TeamPolicy<Threads>,
+             ReducerType, value_type>::Reducer;
+         impl_team_reduce(wrapped_reducer_type(reducer), contribution);
+         reducer.reference() = contribution;))
+  }
 
-        if (m_instance == nullptr) return;
+  template <typename WrappedReducerType>
+  KOKKOS_INLINE_FUNCTION std::enable_if_t<is_reducer<WrappedReducerType>::value>
+  impl_team_reduce(
+      const WrappedReducerType& wrapped_reducer,
+      typename WrappedReducerType::value_type& contribution) const {
+    using value_type = typename WrappedReducerType::value_type;
+    // Make sure there is enough scratch space:
+    using type = std::conditional_t<sizeof(value_type) < TEAM_REDUCE_SIZE,
+                                    value_type, void>;
 
-        type* const local_value = ((type*)m_instance->scratch_memory());
+    type* const local_value = ((type*)m_instance->scratch_memory());
 
-        // Set this thread's contribution
-        if (team_rank() != team_size() - 1) { *local_value = contribution; }
+    // Set this thread's contribution
+    if (team_rank() != team_size() - 1) {
+      *local_value = contribution;
+    }
 
-        // Fence to make sure the base team member has access:
-        memory_fence();
+    // Fence to make sure the base team member has access:
+    memory_fence();
 
-        if (team_fan_in()) {
-          // The last thread to synchronize returns true, all other threads
-          // wait for team_fan_out()
-          type* const team_value = ((type*)m_team_base[0]->scratch_memory());
+    if (team_fan_in()) {
+      // The last thread to synchronize returns true, all other threads
+      // wait for team_fan_out()
+      type* const team_value = ((type*)m_team_base[0]->scratch_memory());
 
-          *team_value = contribution;
-          // Join to the team value:
-          for (int i = 1; i < m_team_size; ++i) {
-            reducer.join(*team_value,
-                         *((type*)m_team_base[i]->scratch_memory()));
-          }
+      *team_value = contribution;
+      // Join to the team value:
+      for (int i = 1; i < m_team_size; ++i) {
+        wrapped_reducer.join(team_value,
+                             ((type*)m_team_base[i]->scratch_memory()));
+      }
 
-          // Team base thread may "lap" member threads so copy out to their
-          // local value.
-          for (int i = 1; i < m_team_size; ++i) {
-            *((type*)m_team_base[i]->scratch_memory()) = *team_value;
-          }
+      // Team base thread may "lap" member threads so copy out to their
+      // local value.
+      for (int i = 1; i < m_team_size; ++i) {
+        *((type*)m_team_base[i]->scratch_memory()) = *team_value;
+      }
 
-          // Fence to make sure all team members have access
-          memory_fence();
-        }
+      // Fence to make sure all team members have access
+      memory_fence();
+    }
 
-        team_fan_out();
+    team_fan_out();
 
-        // Value was changed by the team base
-        reducer.reference() = *local_value;))
+    contribution = *local_value;
   }
 
   /** \brief  Intra-team exclusive prefix sum with team_rank() ordering
@@ -282,10 +292,8 @@ class ThreadsExecTeamMember {
     KOKKOS_IF_ON_DEVICE(((void)global_accum; return value;))
 
     KOKKOS_IF_ON_HOST((  // Make sure there is enough scratch space:
-        using type = typename if_c<sizeof(ArgType) < TEAM_REDUCE_SIZE, ArgType,
-                                   void>::type;
-
-        if (m_instance == nullptr) return type(0);
+        using type = std::conditional_t<sizeof(ArgType) < TEAM_REDUCE_SIZE,
+                                        ArgType, void>;
 
         volatile type* const work_value = ((type*)m_instance->scratch_memory());
 
@@ -358,6 +366,7 @@ class ThreadsExecTeamMember {
         m_chunk_size(team.chunk_size()),
         m_league_chunk_end(0),
         m_team_alloc(team.team_alloc()) {
+    KOKKOS_ASSERT(m_instance != nullptr);
     if (team.league_size()) {
       // Execution is using device-team interface:
 
@@ -418,9 +427,10 @@ class ThreadsExecTeamMember {
         m_instance->set_work_range(m_league_rank, m_league_end, m_chunk_size);
         m_instance->reset_steal_target(m_team_size);
       }
-      if (std::is_same<typename TeamPolicyInternal<
-                           Kokkos::Threads, Properties...>::schedule_type::type,
-                       Kokkos::Dynamic>::value) {
+      if (std::is_same_v<
+              typename TeamPolicyInternal<Kokkos::Threads,
+                                          Properties...>::schedule_type::type,
+              Kokkos::Dynamic>) {
         m_instance->barrier();
       }
     } else {
@@ -757,12 +767,12 @@ class TeamPolicyInternal<Kokkos::Threads, Properties...>
  private:
   /** \brief finalize chunk_size if it was set to AUTO*/
   inline void set_auto_chunk_size() {
-    int64_t concurrency = space().concurrency() / m_team_alloc;
+    int concurrency = space().concurrency() / m_team_alloc;
     if (concurrency == 0) concurrency = 1;
 
-    if (m_chunk_size > 0) {
-      if (!Impl::is_integral_power_of_two(m_chunk_size))
-        Kokkos::abort("TeamPolicy blocking granularity must be power of two");
+    if (m_chunk_size > 0 &&
+        !Kokkos::has_single_bit(static_cast<unsigned>(m_chunk_size))) {
+      Kokkos::abort("TeamPolicy blocking granularity must be power of two");
     }
 
     int new_chunk_size = 1;
@@ -892,19 +902,25 @@ KOKKOS_INLINE_FUNCTION std::enable_if_t<!Kokkos::is_reducer<ValueType>::value>
 parallel_reduce(const Impl::TeamThreadRangeBoundariesStruct<
                     iType, Impl::ThreadsExecTeamMember>& loop_boundaries,
                 const Lambda& lambda, ValueType& result) {
-  ValueType intermediate;
-  Sum<ValueType> sum(intermediate);
-  sum.init(intermediate);
+  using functor_analysis_type = typename Impl::FunctorAnalysis<
+      Impl::FunctorPatternInterface::REDUCE,
+      TeamPolicy<typename Impl::ThreadsExecTeamMember::execution_space>, Lambda,
+      ValueType>;
+  using wrapped_reducer_type = typename functor_analysis_type::Reducer;
+  using value_type           = typename wrapped_reducer_type::value_type;
+
+  wrapped_reducer_type wrapped_reducer(lambda);
+  value_type value;
+  wrapped_reducer.init(&value);
 
   for (iType i = loop_boundaries.start; i < loop_boundaries.end;
        i += loop_boundaries.increment) {
-    ValueType tmp = ValueType();
-    lambda(i, tmp);
-    intermediate += tmp;
+    lambda(i, value);
   }
 
-  loop_boundaries.thread.team_reduce(sum, intermediate);
-  result = sum.reference();
+  loop_boundaries.member.impl_team_reduce(wrapped_reducer, value);
+  wrapped_reducer.final(&value);
+  result = value;
 }
 
 template <typename iType, class Lambda, typename ReducerType>
@@ -912,15 +928,25 @@ KOKKOS_INLINE_FUNCTION std::enable_if_t<Kokkos::is_reducer<ReducerType>::value>
 parallel_reduce(const Impl::TeamThreadRangeBoundariesStruct<
                     iType, Impl::ThreadsExecTeamMember>& loop_boundaries,
                 const Lambda& lambda, const ReducerType& reducer) {
-  typename ReducerType::value_type value;
-  reducer.init(value);
+  using value_type            = typename ReducerType::value_type;
+  using functor_analysis_type = typename Impl::FunctorAnalysis<
+      Impl::FunctorPatternInterface::REDUCE,
+      TeamPolicy<typename Impl::ThreadsExecTeamMember::execution_space>,
+      ReducerType, value_type>;
+  using wrapped_reducer_type = typename functor_analysis_type::Reducer;
+
+  wrapped_reducer_type wrapped_reducer(reducer);
+  value_type value;
+  wrapped_reducer.init(&value);
 
   for (iType i = loop_boundaries.start; i < loop_boundaries.end;
        i += loop_boundaries.increment) {
     lambda(i, value);
   }
 
-  loop_boundaries.thread.team_reduce(reducer, value);
+  loop_boundaries.member.impl_team_reduce(wrapped_reducer, value);
+  wrapped_reducer.final(&value);
+  reducer.reference() = value;
 }
 
 }  // namespace Kokkos
@@ -955,11 +981,24 @@ KOKKOS_INLINE_FUNCTION std::enable_if_t<!Kokkos::is_reducer<ValueType>::value>
 parallel_reduce(const Impl::ThreadVectorRangeBoundariesStruct<
                     iType, Impl::ThreadsExecTeamMember>& loop_boundaries,
                 const Lambda& lambda, ValueType& result) {
-  result = ValueType();
+  using functor_analysis_type = typename Impl::FunctorAnalysis<
+      Impl::FunctorPatternInterface::REDUCE,
+      TeamPolicy<typename Impl::ThreadsExecTeamMember::execution_space>, Lambda,
+      ValueType>;
+  using wrapped_reducer_type = typename functor_analysis_type::Reducer;
+  using value_type           = typename wrapped_reducer_type::value_type;
+
+  wrapped_reducer_type wrapped_reducer(lambda);
+  value_type value;
+  wrapped_reducer.init(&value);
+
   for (iType i = loop_boundaries.start; i < loop_boundaries.end;
        i += loop_boundaries.increment) {
-    lambda(i, result);
+    lambda(i, value);
   }
+
+  wrapped_reducer.final(&value);
+  result = value;
 }
 
 template <typename iType, class Lambda, typename ReducerType>
@@ -967,11 +1006,24 @@ KOKKOS_INLINE_FUNCTION std::enable_if_t<Kokkos::is_reducer<ReducerType>::value>
 parallel_reduce(const Impl::ThreadVectorRangeBoundariesStruct<
                     iType, Impl::ThreadsExecTeamMember>& loop_boundaries,
                 const Lambda& lambda, const ReducerType& reducer) {
-  reducer.init(reducer.reference());
+  using value_type            = typename ReducerType::value_type;
+  using functor_analysis_type = typename Impl::FunctorAnalysis<
+      Impl::FunctorPatternInterface::REDUCE,
+      TeamPolicy<typename Impl::ThreadsExecTeamMember::execution_space>,
+      ReducerType, value_type>;
+  using wrapped_reducer_type = typename functor_analysis_type::Reducer;
+
+  wrapped_reducer_type wrapped_reducer(reducer);
+  value_type value;
+  wrapped_reducer.init(&value);
+
   for (iType i = loop_boundaries.start; i < loop_boundaries.end;
        i += loop_boundaries.increment) {
-    lambda(i, reducer.reference());
+    lambda(i, value);
   }
+
+  wrapped_reducer.final(&value);
+  reducer.reference() = value;
 }
 
 /** \brief  Inter-thread parallel exclusive prefix sum. Executes
@@ -1001,7 +1053,7 @@ KOKKOS_INLINE_FUNCTION void parallel_scan(
     lambda(i, scan_val, false);
   }
 
-  auto& team_member = loop_bounds.thread;
+  auto& team_member = loop_bounds.member;
 
   // 'scan_val' output is the exclusive prefix sum
   scan_val = team_member.team_scan(scan_val);
@@ -1054,7 +1106,7 @@ KOKKOS_INLINE_FUNCTION void parallel_scan(
       typename Impl::FunctorAnalysis<Impl::FunctorPatternInterface::SCAN,
                                      TeamPolicy<Threads>, FunctorType,
                                      void>::value_type;
-  static_assert(std::is_same<closure_value_type, ValueType>::value,
+  static_assert(std::is_same_v<closure_value_type, ValueType>,
                 "Non-matching value types of closure and return type");
 
   ValueType scan_val = ValueType();
