@@ -19,7 +19,6 @@
 
 #if defined(KOKKOS_ENABLE_SYCL)
 namespace Kokkos {
-namespace Experimental {
 class SYCLDeviceUSMSpace;  ///< Memory space on SYCL device, not accessible from
                            ///< the host
 class SYCLSharedUSMSpace;  ///< Memory space accessible from both the SYCL
@@ -27,7 +26,6 @@ class SYCLSharedUSMSpace;  ///< Memory space accessible from both the SYCL
 class SYCLHostUSMSpace;    ///< Memory space accessible from both the SYCL
                            ///< device and the host (host pinned)
 class SYCL;                ///< Execution space for SYCL
-}  // namespace Experimental
 }  // namespace Kokkos
 #endif
 #endif
