@@ -31,37 +31,19 @@
 namespace Kokkos {
 namespace Impl {
 
+template <typename T>
+struct is_graph_capture<
+    T, std::enable_if_t<
+           Kokkos::Impl::is_specialization_of_v<T, GraphNodeCaptureImpl>>>
+    : public std::true_type {};
+
+template <typename T>
+struct is_graph_then_host<
+    T, std::enable_if_t<
+           Kokkos::Impl::is_specialization_of_v<T, GraphNodeThenHostImpl>>>
+    : public std::true_type {};
+
 struct GraphAccess {
-  template <class ExecutionSpace>
-  static Kokkos::Experimental::Graph<ExecutionSpace> construct_graph(
-      ExecutionSpace ex) {
-    //----------------------------------------//
-    return Kokkos::Experimental::Graph<ExecutionSpace>{
-        std::make_shared<GraphImpl<ExecutionSpace>>(std::move(ex))};
-    //----------------------------------------//
-  }
-
-#if defined(KOKKOS_ENABLE_CUDA) || defined(KOKKOS_ENABLE_HIP) || \
-    defined(KOKKOS_ENABLE_SYCL)
-  template <class Exec, typename T>
-  static auto construct_graph_from_native(Exec&& ex, T&& native_graph) {
-    return Kokkos::Experimental::Graph<Kokkos::Impl::remove_cvref_t<Exec>>{
-        std::make_shared<GraphImpl<Kokkos::Impl::remove_cvref_t<Exec>>>(
-            std::forward<Exec>(ex), std::forward<T>(native_graph))};
-  }
-#endif
-
-  template <class ExecutionSpace>
-  static auto create_root_ref(
-      Kokkos::Experimental::Graph<ExecutionSpace>& arg_graph) {
-    auto const& graph_impl_ptr = arg_graph.m_impl_ptr;
-
-    auto root_ptr = graph_impl_ptr->create_root_node_ptr();
-
-    return Kokkos::Experimental::GraphNodeRef<ExecutionSpace>{
-        graph_impl_ptr, std::move(root_ptr)};
-  }
-
   template <class NodeType, class... Args>
   static auto make_node_shared_ptr(Args&&... args) {
     static_assert(
