@@ -87,40 +87,38 @@ KOKKOS_INLINE_FUNCTION
 half_t cast_to_half(half_t);
 
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, float>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, float>, T>
     cast_from_half(half_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, bool>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, bool>, T>
     cast_from_half(half_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, double>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, double>, T>
     cast_from_half(half_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, short>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, short>, T>
     cast_from_half(half_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, int>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, int>, T>
     cast_from_half(half_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, long>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, long>, T>
     cast_from_half(half_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, long long>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, long long>, T>
+    cast_from_half(half_t);
+template <class T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, unsigned short>, T>
+    cast_from_half(half_t);
+template <class T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, unsigned int>, T>
+    cast_from_half(half_t);
+template <class T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, unsigned long>, T>
     cast_from_half(half_t);
 template <class T>
 KOKKOS_INLINE_FUNCTION
-    std::enable_if_t<std::is_same<T, unsigned short>::value, T>
-        cast_from_half(half_t);
-template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, unsigned int>::value, T>
-    cast_from_half(half_t);
-template <class T>
-KOKKOS_INLINE_FUNCTION
-    std::enable_if_t<std::is_same<T, unsigned long>::value, T>
-        cast_from_half(half_t);
-template <class T>
-KOKKOS_INLINE_FUNCTION
-    std::enable_if_t<std::is_same<T, unsigned long long>::value, T>
+    std::enable_if_t<std::is_same_v<T, unsigned long long>, T>
         cast_from_half(half_t);
 
 // declare bhalf_t
@@ -157,40 +155,38 @@ KOKKOS_INLINE_FUNCTION
 bhalf_t cast_to_bhalf(bhalf_t val);
 
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, float>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, float>, T>
     cast_from_bhalf(bhalf_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, bool>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, bool>, T>
     cast_from_bhalf(bhalf_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, double>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, double>, T>
     cast_from_bhalf(bhalf_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, short>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, short>, T>
     cast_from_bhalf(bhalf_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, int>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, int>, T>
     cast_from_bhalf(bhalf_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, long>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, long>, T>
     cast_from_bhalf(bhalf_t);
 template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, long long>::value, T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, long long>, T>
+    cast_from_bhalf(bhalf_t);
+template <class T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, unsigned short>, T>
+    cast_from_bhalf(bhalf_t);
+template <class T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, unsigned int>, T>
+    cast_from_bhalf(bhalf_t);
+template <class T>
+KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same_v<T, unsigned long>, T>
     cast_from_bhalf(bhalf_t);
 template <class T>
 KOKKOS_INLINE_FUNCTION
-    std::enable_if_t<std::is_same<T, unsigned short>::value, T>
-        cast_from_bhalf(bhalf_t);
-template <class T>
-KOKKOS_INLINE_FUNCTION std::enable_if_t<std::is_same<T, unsigned int>::value, T>
-    cast_from_bhalf(bhalf_t);
-template <class T>
-KOKKOS_INLINE_FUNCTION
-    std::enable_if_t<std::is_same<T, unsigned long>::value, T>
-        cast_from_bhalf(bhalf_t);
-template <class T>
-KOKKOS_INLINE_FUNCTION
-    std::enable_if_t<std::is_same<T, unsigned long long>::value, T>
+    std::enable_if_t<std::is_same_v<T, unsigned long long>, T>
         cast_from_bhalf(bhalf_t);
 #endif  // KOKKOS_IMPL_BHALF_TYPE_DEFINED
 
@@ -490,7 +486,7 @@ class alignas(FloatType) floating_point_wrapper {
   // Compound operators: upcast overloads for +=
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator+=(T& lhs, floating_point_wrapper rhs) {
     lhs += static_cast<T>(rhs);
     return lhs;
@@ -526,7 +522,7 @@ class alignas(FloatType) floating_point_wrapper {
   // Compund operators: upcast overloads for -=
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator-=(T& lhs, floating_point_wrapper rhs) {
     lhs -= static_cast<T>(rhs);
     return lhs;
@@ -562,7 +558,7 @@ class alignas(FloatType) floating_point_wrapper {
   // Compund operators: upcast overloads for *=
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator*=(T& lhs, floating_point_wrapper rhs) {
     lhs *= static_cast<T>(rhs);
     return lhs;
@@ -598,7 +594,7 @@ class alignas(FloatType) floating_point_wrapper {
   // Compund operators: upcast overloads for /=
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator/=(T& lhs, floating_point_wrapper rhs) {
     lhs /= static_cast<T>(rhs);
     return lhs;
@@ -636,14 +632,14 @@ class alignas(FloatType) floating_point_wrapper {
   // Binary Arithmetic upcast operators for +
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator+(floating_point_wrapper lhs, T rhs) {
     return T(lhs) + rhs;
   }
 
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator+(T lhs, floating_point_wrapper rhs) {
     return lhs + T(rhs);
   }
@@ -665,14 +661,14 @@ class alignas(FloatType) floating_point_wrapper {
   // Binary Arithmetic upcast operators for -
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator-(floating_point_wrapper lhs, T rhs) {
     return T(lhs) - rhs;
   }
 
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator-(T lhs, floating_point_wrapper rhs) {
     return lhs - T(rhs);
   }
@@ -694,14 +690,14 @@ class alignas(FloatType) floating_point_wrapper {
   // Binary Arithmetic upcast operators for *
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator*(floating_point_wrapper lhs, T rhs) {
     return T(lhs) * rhs;
   }
 
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator*(T lhs, floating_point_wrapper rhs) {
     return lhs * T(rhs);
   }
@@ -723,14 +719,14 @@ class alignas(FloatType) floating_point_wrapper {
   // Binary Arithmetic upcast operators for /
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator/(floating_point_wrapper lhs, T rhs) {
     return T(lhs) / rhs;
   }
 
   template <class T>
   KOKKOS_FUNCTION friend std::enable_if_t<
-      std::is_same<T, float>::value || std::is_same<T, double>::value, T>
+      std::is_same_v<T, float> || std::is_same_v<T, double>, T>
   operator/(T lhs, floating_point_wrapper rhs) {
     return lhs / T(rhs);
   }
@@ -987,13 +983,12 @@ half_t cast_to_half(unsigned long long val) { return half_t(val); }
 // example don't include char
 template <class T>
 KOKKOS_INLINE_FUNCTION std::enable_if_t<
-    std::is_same<T, float>::value || std::is_same<T, bool>::value ||
-        std::is_same<T, double>::value || std::is_same<T, short>::value ||
-        std::is_same<T, unsigned short>::value || std::is_same<T, int>::value ||
-        std::is_same<T, unsigned int>::value || std::is_same<T, long>::value ||
-        std::is_same<T, unsigned long>::value ||
-        std::is_same<T, long long>::value ||
-        std::is_same<T, unsigned long long>::value,
+    std::is_same_v<T, float> || std::is_same_v<T, bool> ||
+        std::is_same_v<T, double> || std::is_same_v<T, short> ||
+        std::is_same_v<T, unsigned short> || std::is_same_v<T, int> ||
+        std::is_same_v<T, unsigned int> || std::is_same_v<T, long> ||
+        std::is_same_v<T, unsigned long> || std::is_same_v<T, long long> ||
+        std::is_same_v<T, unsigned long long>,
     T>
 cast_from_half(half_t val) {
   return T(val);
@@ -1047,13 +1042,12 @@ bhalf_t cast_to_bhalf(unsigned long long val) { return bhalf_t(val); }
 // cast_from_bhalf
 template <class T>
 KOKKOS_INLINE_FUNCTION std::enable_if_t<
-    std::is_same<T, float>::value || std::is_same<T, bool>::value ||
-        std::is_same<T, double>::value || std::is_same<T, short>::value ||
-        std::is_same<T, unsigned short>::value || std::is_same<T, int>::value ||
-        std::is_same<T, unsigned int>::value || std::is_same<T, long>::value ||
-        std::is_same<T, unsigned long>::value ||
-        std::is_same<T, long long>::value ||
-        std::is_same<T, unsigned long long>::value,
+    std::is_same_v<T, float> || std::is_same_v<T, bool> ||
+        std::is_same_v<T, double> || std::is_same_v<T, short> ||
+        std::is_same_v<T, unsigned short> || std::is_same_v<T, int> ||
+        std::is_same_v<T, unsigned int> || std::is_same_v<T, long> ||
+        std::is_same_v<T, unsigned long> || std::is_same_v<T, long long> ||
+        std::is_same_v<T, unsigned long long>,
     T>
 cast_from_bhalf(bhalf_t val) {
   return T(val);
