@@ -23,7 +23,7 @@ using ScalarType     = double;
 using ViewType       = Kokkos::View<ScalarType*, ExecutionSpace>;
 using ViewTypeHost   = Kokkos::View<ScalarType*, Kokkos::HostSpace>;
 KOKKOS_FUNCTION
-const half_t& accept_ref(const half_t& a) { return a; }
+half_t accept_ref(const half_t& a) { return a; }
 KOKKOS_FUNCTION
 double accept_ref_expected(const half_t& a) {
   double tmp = static_cast<double>(a);
@@ -31,7 +31,7 @@ double accept_ref_expected(const half_t& a) {
 }
 #if !KOKKOS_BHALF_T_IS_FLOAT
 KOKKOS_FUNCTION
-const bhalf_t& accept_ref(const bhalf_t& a) { return a; }
+bhalf_t accept_ref(const bhalf_t& a) { return a; }
 KOKKOS_FUNCTION
 double accept_ref_expected(const bhalf_t& a) {
   double tmp = static_cast<double>(a);
@@ -294,7 +294,7 @@ struct Functor_TestHalfOperators {
     d_lhs        = static_cast<double>(h_lhs);
     d_rhs        = static_cast<double>(h_rhs);
 
-    if (std::is_same<view_type, ViewTypeHost>::value) {
+    if (std::is_same_v<view_type, ViewTypeHost>) {
       auto run_on_host = *this;
       run_on_host(Batch0{}, 0);
       run_on_host(Batch1{}, 0);
@@ -331,13 +331,13 @@ struct Functor_TestHalfOperators {
     auto sum = static_cast<LhsType>(h_lhs) + static_cast<RhsType>(h_rhs);
     actual_lhs(op_test_idx) = static_cast<double>(sum);
 
-    if (std::is_same<RhsType, half_type>::value &&
-        std::is_same<LhsType, half_type>::value) {
+    if (std::is_same_v<RhsType, half_type> &&
+        std::is_same_v<LhsType, half_type>) {
       expected_lhs(op_test_idx) = d_lhs + d_rhs;
     } else {
-      if (std::is_same<LhsType, half_type>::value)
+      if (std::is_same_v<LhsType, half_type>)
         expected_lhs(op_test_idx) = d_lhs + static_cast<RhsType>(d_rhs);
-      if (std::is_same<RhsType, half_type>::value)
+      if (std::is_same_v<RhsType, half_type>)
         expected_lhs(op_test_idx) = static_cast<LhsType>(d_lhs) + d_rhs;
     }
 
@@ -351,13 +351,13 @@ struct Functor_TestHalfOperators {
     auto result = static_cast<LhsType>(h_lhs) - static_cast<RhsType>(h_rhs);
     actual_lhs(op_test_idx) = static_cast<double>(result);
 
-    if (std::is_same<RhsType, half_type>::value &&
-        std::is_same<LhsType, half_type>::value) {
+    if (std::is_same_v<RhsType, half_type> &&
+        std::is_same_v<LhsType, half_type>) {
       expected_lhs(op_test_idx) = d_lhs - d_rhs;
     } else {
-      if (std::is_same<LhsType, half_type>::value)
+      if (std::is_same_v<LhsType, half_type>)
         expected_lhs(op_test_idx) = d_lhs - static_cast<RhsType>(d_rhs);
-      if (std::is_same<RhsType, half_type>::value)
+      if (std::is_same_v<RhsType, half_type>)
         expected_lhs(op_test_idx) = static_cast<LhsType>(d_lhs) - d_rhs;
     }
 
@@ -371,13 +371,13 @@ struct Functor_TestHalfOperators {
     auto result = static_cast<LhsType>(h_lhs) * static_cast<RhsType>(h_rhs);
     actual_lhs(op_test_idx) = static_cast<double>(result);
 
-    if (std::is_same<RhsType, half_type>::value &&
-        std::is_same<LhsType, half_type>::value) {
+    if (std::is_same_v<RhsType, half_type> &&
+        std::is_same_v<LhsType, half_type>) {
       expected_lhs(op_test_idx) = d_lhs * d_rhs;
     } else {
-      if (std::is_same<LhsType, half_type>::value)
+      if (std::is_same_v<LhsType, half_type>)
         expected_lhs(op_test_idx) = d_lhs * static_cast<RhsType>(d_rhs);
-      if (std::is_same<RhsType, half_type>::value)
+      if (std::is_same_v<RhsType, half_type>)
         expected_lhs(op_test_idx) = static_cast<LhsType>(d_lhs) * d_rhs;
     }
 
@@ -391,13 +391,13 @@ struct Functor_TestHalfOperators {
     auto result = static_cast<LhsType>(h_lhs) / static_cast<RhsType>(h_rhs);
     actual_lhs(op_test_idx) = static_cast<double>(result);
 
-    if (std::is_same<RhsType, half_type>::value &&
-        std::is_same<LhsType, half_type>::value) {
+    if (std::is_same_v<RhsType, half_type> &&
+        std::is_same_v<LhsType, half_type>) {
       expected_lhs(op_test_idx) = d_lhs / d_rhs;
     } else {
-      if (std::is_same<LhsType, half_type>::value)
+      if (std::is_same_v<LhsType, half_type>)
         expected_lhs(op_test_idx) = d_lhs / static_cast<RhsType>(d_rhs);
-      if (std::is_same<RhsType, half_type>::value)
+      if (std::is_same_v<RhsType, half_type>)
         expected_lhs(op_test_idx) = static_cast<LhsType>(d_lhs) / d_rhs;
     }
 
@@ -965,7 +965,7 @@ struct Functor_TestHalfOperators {
 };
 
 template <class half_type>
-void __test_half_operators(half_type h_lhs, half_type h_rhs) {
+void _test_half_operators(half_type h_lhs, half_type h_rhs) {
   half_type epsilon = Kokkos::Experimental::epsilon<half_type>::value;
 
   Functor_TestHalfOperators<ViewType, half_type> f_device(h_lhs, h_rhs);
@@ -1017,10 +1017,10 @@ void test_half_operators() {
   for (int i = -3; i < 2; i++) {
     // printf("%f OP %f\n", float(h_lhs + cast_to_half(i + 1)), float(h_rhs +
     // cast_to_half(i)));
-    __test_half_operators<half_t>(h_lhs + cast_to_half(i + 1),
-                                  h_rhs + cast_to_half(i));
-    // TODO: __test_half_operators(h_lhs + cast_to_half(i + 1), half_t(0));
-    // TODO: __test_half_operators(half_t(0), h_rhs + cast_to_half(i));
+    _test_half_operators<half_t>(h_lhs + cast_to_half(i + 1),
+                                 h_rhs + cast_to_half(i));
+    // TODO: _test_half_operators(h_lhs + cast_to_half(i + 1), half_t(0));
+    // TODO: _test_half_operators(half_t(0), h_rhs + cast_to_half(i));
   }
 }
 
@@ -1029,8 +1029,8 @@ void test_bhalf_operators() {
   for (int i = -2; i < 2; i++) {
     // printf("%f OP %f\n", float(h_lhs + cast_to_bhalf(i + 1)), float(h_rhs +
     // cast_to_bhalf(i)));
-    __test_half_operators<bhalf_t>(h_lhs + cast_to_bhalf(i + 1),
-                                   h_rhs + cast_to_bhalf(i));
+    _test_half_operators<bhalf_t>(h_lhs + cast_to_bhalf(i + 1),
+                                  h_rhs + cast_to_bhalf(i));
   }
 }
 
