@@ -15,5 +15,7 @@
 //@HEADER
 
 #define KOKKOS_MATHEMATICAL_FUNCTIONS_SKIP_1
+#define KOKKOS_MATHEMATICAL_FUNCTIONS_SKIP_3
 #include "TestMathematicalFunctions.hpp"
 #undef KOKKOS_MATHEMATICAL_FUNCTIONS_SKIP_1
+#undef KOKKOS_MATHEMATICAL_FUNCTIONS_SKIP_3
