@@ -16,13 +16,10 @@
 
 #ifndef KOKKOS_TEST_OTHER_HPP
 #define KOKKOS_TEST_OTHER_HPP
-#include <TestAggregate.hpp>
 #include <TestMemoryPool.hpp>
 #include <TestCXX11.hpp>
 
+#ifdef KOKKOS_ENABLE_IMPL_VIEW_LEGACY
 #include <TestViewCtorPropEmbeddedDim.hpp>
-// with VS 16.11.3 and CUDA 11.4.2 getting cudafe stackoverflow crash
-#if !(defined(_WIN32) && defined(KOKKOS_ENABLE_CUDA))
-#include <TestViewLayoutTiled.hpp>
 #endif
 #endif
