@@ -57,14 +57,14 @@ TEST(hip, preferred_blocksize_deduction) {
 
   {
     using DriverType = Kokkos::Impl::ParallelFor<TestNone, policy>;
-    ASSERT_TRUE(
-        Kokkos::Impl::HIPParallelLaunch<DriverType>::get_scratch_size() == 0);
+    ASSERT_TRUE(Kokkos::Impl::HIPParallelLaunch<DriverType>::get_scratch_size(
+                    execution_space().hip_device()) == 0);
   }
 
   {
     using DriverType = Kokkos::Impl::ParallelFor<TestSpiller, policy>;
-    ASSERT_TRUE(
-        Kokkos::Impl::HIPParallelLaunch<DriverType>::get_scratch_size() > 0);
+    ASSERT_TRUE(Kokkos::Impl::HIPParallelLaunch<DriverType>::get_scratch_size(
+                    execution_space().hip_device()) > 0);
   }
 }
 
