@@ -17,18 +17,20 @@
 #include <Kokkos_Core.hpp>
 #include <TestHPX_Category.hpp>
 
-#include <hpx/local/future.hpp>
-
-#ifdef KOKKOS_ENABLE_HPX_ASYNC_DISPATCH
+#include <hpx/execution.hpp>
 
 namespace {
 
 TEST(hpx, independent_instances_delayed_execution) {
   Kokkos::View<bool, Kokkos::Experimental::HPX> ran("ran");
-  hpx::lcos::local::promise<void> p;
-  hpx::shared_future<void> f = p.get_future();
 
-  Kokkos::Experimental::HPX hpx(f);
+  // Create a sender that will call set_value on a receiver after a delay.
+  hpx::execution::experimental::unique_any_sender<> s{
+      hpx::execution::experimental::schedule(
+          hpx::execution::experimental::thread_pool_scheduler{}) |
+      hpx::execution::experimental::then(
+          [] { hpx::this_thread::sleep_for(std::chrono::milliseconds(500)); })};
+  Kokkos::Experimental::HPX hpx(std::move(s));
   Kokkos::parallel_for(
       "Test::hpx::independent_instances::delay_execution",
       Kokkos::Experimental::require(
@@ -36,15 +38,13 @@ TEST(hpx, independent_instances_delayed_execution) {
           Kokkos::Experimental::WorkItemProperty::HintLightWeight),
       KOKKOS_LAMBDA(int) { ran() = true; });
 
+#if defined(KOKKOS_ENABLE_IMPL_HPX_ASYNC_DISPATCH)
   ASSERT_FALSE(ran());
-  ASSERT_FALSE(hpx.impl_get_future().is_ready());
-
-  p.set_value();
-
+#else
+  ASSERT_TRUE(ran());
+#endif
   hpx.fence();
-  ASSERT_TRUE(hpx.impl_get_future().is_ready());
+  ASSERT_TRUE(ran());
 }
 
 }  // namespace
-
-#endif
