@@ -33,10 +33,10 @@ template <class ExecSpace>
 struct TestIncrExecSpaceTypedef {
   void testit() {
     const bool passed =
-        (!std::is_void<typename ExecSpace::memory_space>::value) &&
-        std::is_same<ExecSpace, typename ExecSpace::execution_space>::value &&
-        !std::is_void<typename ExecSpace::scratch_memory_space>::value &&
-        !std::is_void<typename ExecSpace::array_layout>::value;
+        (!std::is_void_v<typename ExecSpace::memory_space>)&&std::is_same_v<
+            ExecSpace, typename ExecSpace::execution_space> &&
+        !std::is_void_v<typename ExecSpace::scratch_memory_space> &&
+        !std::is_void_v<typename ExecSpace::array_layout>;
     static_assert(passed == true,
                   "The memory and execution spaces are defined");
   }
@@ -50,8 +50,8 @@ struct TestIncrExecSpace {
     using execution_space = typename device_type::execution_space;
 
     const bool passed =
-        std::is_same<device_type,
-                     Kokkos::Device<execution_space, memory_space>>::value;
+        std::is_same_v<device_type,
+                       Kokkos::Device<execution_space, memory_space>>;
 
     static_assert(passed == true,
                   "Checking if the is_execution_space is evaluated correctly");
@@ -63,7 +63,13 @@ struct TestIncrExecSpace {
     ASSERT_GT(concurrency, 0);
 
 #ifdef KOKKOS_ENABLE_DEPRECATED_CODE_4
+#ifdef KOKKOS_ENABLE_DEPRECATION_WARNINGS
+    KOKKOS_IMPL_DISABLE_DEPRECATED_WARNINGS_PUSH()
+#endif
     int in_parallel = ExecSpace::in_parallel();
+#ifdef KOKKOS_ENABLE_DEPRECATION_WARNINGS
+    KOKKOS_IMPL_DISABLE_DEPRECATED_WARNINGS_POP()
+#endif
     ASSERT_FALSE(in_parallel);
 #endif
 
