@@ -22,17 +22,16 @@ namespace Test {
 namespace Impl {
 
 struct SYCLQueueScratchTestFunctor {
-  using team_t = Kokkos::TeamPolicy<Kokkos::Experimental::SYCL>::member_type;
-  using scratch_t =
-      Kokkos::View<int64_t*, Kokkos::Experimental::SYCL::scratch_memory_space>;
+  using team_t    = Kokkos::TeamPolicy<Kokkos::SYCL>::member_type;
+  using scratch_t = Kokkos::View<int64_t*, Kokkos::SYCL::scratch_memory_space>;
 
-  Kokkos::View<int64_t, Kokkos::Experimental::SYCLDeviceUSMSpace,
+  Kokkos::View<int64_t, Kokkos::SYCLDeviceUSMSpace,
                Kokkos::MemoryTraits<Kokkos::Atomic>>
       counter;
   int N, M;
   SYCLQueueScratchTestFunctor(
-      Kokkos::View<int64_t, Kokkos::Experimental::SYCLDeviceUSMSpace> counter_,
-      int N_, int M_)
+      Kokkos::View<int64_t, Kokkos::SYCLDeviceUSMSpace> counter_, int N_,
+      int M_)
       : counter(counter_), N(N_), M(M_) {}
 
   KOKKOS_FUNCTION
@@ -54,12 +53,11 @@ struct SYCLQueueScratchTestFunctor {
 
 void sycl_queue_scratch_test_one(
     int N, int T, int M_base,
-    Kokkos::View<int64_t, Kokkos::Experimental::SYCLDeviceUSMSpace> counter,
-    Kokkos::Experimental::SYCL sycl, int tid) {
+    Kokkos::View<int64_t, Kokkos::SYCLDeviceUSMSpace> counter,
+    Kokkos::SYCL sycl, int tid) {
   int M = M_base + tid * 5;
-  Kokkos::TeamPolicy<Kokkos::Experimental::SYCL> p(sycl, T, 64);
-  using scratch_t =
-      Kokkos::View<int64_t*, Kokkos::Experimental::SYCL::scratch_memory_space>;
+  Kokkos::TeamPolicy<Kokkos::SYCL> p(sycl, T, 64);
+  using scratch_t = Kokkos::View<int64_t*, Kokkos::SYCL::scratch_memory_space>;
 
   int bytes = scratch_t::shmem_size(M);
 
@@ -71,18 +69,19 @@ void sycl_queue_scratch_test_one(
 
 void sycl_queue_scratch_test(
     int N, int T, int M_base,
-    Kokkos::View<int64_t, Kokkos::Experimental::SYCLDeviceUSMSpace> counter) {
+    Kokkos::View<int64_t, Kokkos::SYCLDeviceUSMSpace> counter) {
   constexpr int K = 4;
-  Kokkos::Experimental::SYCL default_space;
+  Kokkos::SYCL default_space;
   sycl::context default_context = default_space.sycl_queue().get_context();
 
-  sycl::default_selector device_selector;
-  sycl::queue queue(default_context, device_selector);
+  sycl::queue queue(default_context, sycl::default_selector_v,
+                    sycl::property::queue::in_order());
 
-  std::array<Kokkos::Experimental::SYCL, K> sycl;
+  std::array<Kokkos::SYCL, K> sycl;
   for (int i = 0; i < K; i++) {
-    sycl[i] = Kokkos::Experimental::SYCL(
-        sycl::queue(default_context, device_selector));
+    sycl[i] =
+        Kokkos::SYCL(sycl::queue(default_context, sycl::default_selector_v,
+                                 sycl::property::queue::in_order()));
   }
 
   // Test that growing scratch size in subsequent calls doesn't crash things
@@ -115,7 +114,7 @@ TEST(sycl, team_scratch_1_queues) {
   int T      = 10;
   int M_base = 150;
 
-  Kokkos::View<int64_t, Kokkos::Experimental::SYCLDeviceUSMSpace> counter("C");
+  Kokkos::View<int64_t, Kokkos::SYCLDeviceUSMSpace> counter("C");
 
   Impl::sycl_queue_scratch_test(N, T, M_base, counter);
 
