@@ -18,16 +18,7 @@
 #define KOKKOS_SIMD_HPP
 
 #include <Kokkos_SIMD_Common.hpp>
-
-// suppress NVCC warnings with the [[nodiscard]] attribute on overloaded
-// operators implemented as hidden friends
-#if defined(KOKKOS_COMPILER_NVCC) && KOKKOS_COMPILER_NVCC < 1130
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wattributes"
-#endif
-
 #include <Kokkos_SIMD_Scalar.hpp>
-
 #include <Kokkos_Macros.hpp>
 
 // FIXME_OPENMPTARGET The device pass disables all compiler macros checked
@@ -40,6 +31,10 @@
 #include <Kokkos_SIMD_AVX512.hpp>
 #endif
 
+#if defined(KOKKOS_ARCH_ARM_SVE)
+#include <Kokkos_SIMD_SVE.hpp>
+#endif
+
 #if defined(KOKKOS_ARCH_ARM_NEON)
 #include <Kokkos_SIMD_NEON.hpp>
 #endif
@@ -62,18 +57,22 @@
 #include <Kokkos_SIMD_AVX512.hpp>
 #endif
 
+#if defined(KOKKOS_ARCH_ARM_SVE)
+#if !defined(__ARM_FEATURE_SVE_BITS) || !defined(__ARM_NEON)
+#error \
+    "Both __ARM_FEATURE_SVE_BITS and __ARM_NEON must be definded for KOKKOS_ARCH_ARM_SVE"
+#endif
+#include <Kokkos_SIMD_SVE.hpp>
+#endif
+
 #if defined(KOKKOS_ARCH_ARM_NEON)
 #if !defined(__ARM_NEON)
-#error "__ARM_NEON must be definded for KOKKOS_ARCH_ARM_NEON"
+#error "__ARM_NEON must be defined for KOKKOS_ARCH_ARM_NEON"
 #endif
 #include <Kokkos_SIMD_NEON.hpp>
 #endif
 #endif
 
-#if defined(KOKKOS_COMPILER_NVCC) && KOKKOS_COMPILER_NVCC < 1130
-#pragma GCC diagnostic pop
-#endif
-
 #include <Kokkos_SIMD_Common_Math.hpp>
 
 namespace Kokkos {
@@ -84,33 +83,45 @@ namespace simd_abi {
 namespace Impl {
 
 #if defined(KOKKOS_ARCH_AVX512XEON)
+template <class T>
 using host_fixed_native = avx512_fixed_size<8>;
 template <int N>
 using host_native_abi = avx512_fixed_size<N>;
 
 #elif defined(KOKKOS_ARCH_AVX2)
+template <class T>
 using host_fixed_native = avx2_fixed_size<4>;
 template <int N>
 using host_native_abi = avx2_fixed_size<N>;
 
+#elif defined(KOKKOS_ARCH_ARM_SVE)
+template <class T>
+using host_fixed_native =
+    sve_fixed_size<(__ARM_FEATURE_SVE_BITS / (8 * sizeof(T)))>;
+template <int N>
+using host_native_abi = sve_fixed_size<N>;
+
 #elif defined(KOKKOS_ARCH_ARM_NEON)
+template <class T>
 using host_fixed_native = neon_fixed_size<2>;
 template <int N>
 using host_native_abi = neon_fixed_size<N>;
 
 #else
+template <class T>
 using host_fixed_native = scalar;
 template <int N>
 using host_native_abi = scalar;
 #endif
 
-template <class T>
+template <class S>
 struct ForSpace;
 
 #ifdef KOKKOS_ENABLE_SERIAL
 template <>
 struct ForSpace<Kokkos::Serial> {
-  using type = host_fixed_native;
+  template <class T>
+  using type = host_fixed_native<T>;
 
   template <int N>
   using simd_abi = host_native_abi<N>;
@@ -120,6 +131,7 @@ struct ForSpace<Kokkos::Serial> {
 #ifdef KOKKOS_ENABLE_CUDA
 template <>
 struct ForSpace<Kokkos::Cuda> {
+  template <class T>
   using type = scalar;
 
   template <int N>
@@ -130,7 +142,8 @@ struct ForSpace<Kokkos::Cuda> {
 #ifdef KOKKOS_ENABLE_THREADS
 template <>
 struct ForSpace<Kokkos::Threads> {
-  using type = host_fixed_native;
+  template <class T>
+  using type = host_fixed_native<T>;
 
   template <int N>
   using simd_abi = host_native_abi<N>;
@@ -140,6 +153,7 @@ struct ForSpace<Kokkos::Threads> {
 #ifdef KOKKOS_ENABLE_HPX
 template <>
 struct ForSpace<Kokkos::Experimental::HPX> {
+  template <class T>
   using type = scalar;
 
   template <int N>
@@ -150,7 +164,8 @@ struct ForSpace<Kokkos::Experimental::HPX> {
 #ifdef KOKKOS_ENABLE_OPENMP
 template <>
 struct ForSpace<Kokkos::OpenMP> {
-  using type = host_fixed_native;
+  template <class T>
+  using type = host_fixed_native<T>;
 
   template <int N>
   using simd_abi = host_native_abi<N>;
@@ -160,6 +175,7 @@ struct ForSpace<Kokkos::OpenMP> {
 #ifdef KOKKOS_ENABLE_OPENMPTARGET
 template <>
 struct ForSpace<Kokkos::Experimental::OpenMPTarget> {
+  template <class T>
   using type = scalar;
 
   template <int N>
@@ -170,6 +186,7 @@ struct ForSpace<Kokkos::Experimental::OpenMPTarget> {
 #ifdef KOKKOS_ENABLE_OPENACC
 template <>
 struct ForSpace<Kokkos::Experimental::OpenACC> {
+  template <class T>
   using type = scalar;
 
   template <int N>
@@ -180,6 +197,7 @@ struct ForSpace<Kokkos::Experimental::OpenACC> {
 #ifdef KOKKOS_ENABLE_HIP
 template <>
 struct ForSpace<Kokkos::HIP> {
+  template <class T>
   using type = scalar;
 
   template <int N>
@@ -190,6 +208,7 @@ struct ForSpace<Kokkos::HIP> {
 #ifdef KOKKOS_ENABLE_SYCL
 template <>
 struct ForSpace<Kokkos::SYCL> {
+  template <class T>
   using type = scalar;
 
   template <int N>
@@ -197,8 +216,8 @@ struct ForSpace<Kokkos::SYCL> {
 };
 #endif
 
-template <class Space = Kokkos::DefaultExecutionSpace>
-using native_fixed_abi = typename ForSpace<Space>::type;
+template <class T, class Space = Kokkos::DefaultExecutionSpace>
+using native_fixed_abi = typename ForSpace<Space>::template type<T>;
 
 template <int N, class Space = Kokkos::DefaultExecutionSpace>
 using native_abi = typename ForSpace<Space>::template simd_abi<N>;
@@ -206,11 +225,12 @@ using native_abi = typename ForSpace<Space>::template simd_abi<N>;
 }  // namespace Impl
 
 #ifdef KOKKOS_ENABLE_DEPRECATED_CODE_4
-template <class Space>
-using ForSpace = typename Impl::ForSpace<typename Space::execution_space>::type;
+template <class T, class Space>
+using ForSpace =
+    typename Impl::ForSpace<typename Space::execution_space>::template type<T>;
 
 template <class T>
-using native = ForSpace<Kokkos::DefaultExecutionSpace>;
+using native = ForSpace<T, Kokkos::DefaultExecutionSpace>;
 #endif
 
 }  // namespace simd_abi
@@ -227,12 +247,12 @@ using native_simd_mask KOKKOS_DEPRECATED_WITH_COMMENT(
 template <class T, int N = 0>
 using simd =
     basic_simd<T,
-               std::conditional_t<(N == 0), simd_abi::Impl::native_fixed_abi<>,
+               std::conditional_t<(N == 0), simd_abi::Impl::native_fixed_abi<T>,
                                   simd_abi::Impl::native_abi<N>>>;
 
 template <class T, int N = 0>
 using simd_mask = basic_simd_mask<
-    T, std::conditional_t<(N == 0), simd_abi::Impl::native_fixed_abi<>,
+    T, std::conditional_t<(N == 0), simd_abi::Impl::native_fixed_abi<T>,
                           simd_abi::Impl::native_abi<N>>>;
 
 namespace Impl {
@@ -253,6 +273,13 @@ using host_abi_set    = abi_set<simd_abi::scalar, simd_abi::avx2_fixed_size<4>,
                              simd_abi::avx2_fixed_size<8>>;
 using data_type_set =
     data_types<std::int32_t, std::int64_t, std::uint64_t, double, float>;
+#elif defined(KOKKOS_ARCH_ARM_SVE)
+using host_abi_set =
+    abi_set<simd_abi::scalar, simd_abi::sve_fixed_size<2>,
+            simd_abi::sve_fixed_size<4>, simd_abi::sve_fixed_size<8>,
+            simd_abi::sve_fixed_size<16>>;
+using data_type_set = data_types<std::int32_t, std::uint32_t, std::int64_t,
+                                 std::uint64_t, double, float>;
 #elif defined(KOKKOS_ARCH_ARM_NEON)
 using host_abi_set    = abi_set<simd_abi::scalar, simd_abi::neon_fixed_size<2>,
                              simd_abi::neon_fixed_size<4>>;
